#ifndef DEF_GUI_CLASS
#define DEF_GUI_CLASS

/**
* \file GUI.hpp
* \author GUIllaume.C
* \brief main hpp file of the workpackage GUI. Definition of GUI class.
* This file must be included by any other file who uses the PC GUI,
* and should be the only one included.
**/

#include <vector>
#include <exception>

#include "Background.hpp"
#include "Button.hpp"
#include "Message.hpp"
#include "Reseau.hpp"
#include "StatManager.hpp"
#include "SoundManager.hpp"
#include "SliderButton.hpp"
#include "TrophiesDrawer.hpp"


/**
*    \brief Main class of the Work Package PC Client.
*
*    Several cpp are bound to this class. One cpp correspond to one event loop.
*    See files in the Loops folder for more details.
*
**/
class GUI {
public:
    /** \brief GUI constructor : instanciate a new GUI. No parameters are needed **/
    GUI();

    /** \brief Runs the GUI loop. Fonctions defined in GUI.cpp **/
    void run();

    /** \brief Temporary run function in order to perform tests
    *           Added by Guillaume. Don't forget to remove at release
    **/
    void run_test();

private:
    enum class ConnectionType : int {
        CREATE,
        JOIN,
        MATCHMAKING
    };

    enum class Function : int {
        NONE,
        QUIT,
        OPTIONS_LOOP,
        GRAPHIC_OPTION_LOOP,
        GAME_OPTION_LOOP,
        CREDITS,
        MODE_SELECTION,
        GAME_LOOP,
        MULTIPLAYER_SELECTION,
        MULTI_JOIN_GAME,
        MULTI_CREATE_GAME,
        MULTI_FIND_GAME,
        TUTORIAL_SELECTION,
        TUTORIAL_CARDS_LOOP,
        TUTORIAL_TRICKS_LOOP,
        TUTORIAL_BET_LOOP,
        TUTORIAL_FLOW_LOOP
    };

    sf::RenderWindow app;
    Background m_splashScreen;
    Network::ServerCommunication m_serverLink;
    TrophiesDrawer m_trophiesDrawer;
    bool m_splashActive;

    void choose_functionnality(Function const);

    void run_loop(std::vector<Message> &messages,
                  std::vector<TextButton> &text_buttons,
                  std::vector<GraphicButton> &graphic_buttons,
                  std::vector<Function> &functionnalities);

    // These methods are only called during GUI's run method's execution
    void run_game_loop();
    void run_multiplayer_game_loop(ConnectionType const, short const serverID=0);

    void run_login();
    void run_main_menu();
    void run_multiplayer_selection();
    void run_multiplayer_lobby();

    void run_tutorial_loop(std::string path);
    void run_tutorial_card_loop();
    void run_tutorial_menu();

    void run_tuto_cards_loop();
    void run_tuto_tricks_loop();
    void run_tuto_bets_loop();
    void run_tuto_flow_loop();

    void run_options_loop();
    void run_graphic_option_loop();
    void run_game_option_loop();

    void run_credits_loop();
    
    void run_mode_selection_loop();


    void run_stat_loop();
};

#endif
