#ifndef DEF_BET_ARRAY
#define DEF_BET_ARRAY

#include <vector>
#include "Token.hpp"
#include "Core.hpp"

/** \file BetArray.hpp
*    \author Mathieu.V
*    \brief Definition of class BetArray
**/


/** \brief The object that permits the player to choose his bet **/
class BetArray{
public:
    BetArray() = default;

    BetArray(int x, int y, int nb_token);
    void draw(sf::RenderWindow &window);

    /** \brief Sets the number of visible tokens. Setter for the m_nb_tokens attribute **/
    void set_nb_token(int newNbToken);
    /** \brief Sets values used to know what are the playable bets **/
    void set_playable_bet(Core::Bet current_bet, bool mechoune, int nb_forbidden);
    /** \brief Sets bets want by tuto **/
    void set_playable_bet(std::vector<Core::Bet> bets_available);

    /** \brief Gets the token whose hitbox is intersected by the given position
    *   \return The id of the token. Returns -1 if no token is intersected
    **/
    int get_token_at_pos(int posX, int posY);
    int get_token_at_pos(sf::Vector2f const );
    int get_token_at_pos(sf::Vector2i const );

    /** \brief Gets the trump whose hitbox is intersected by the given position
    *   \return The id of the token. Returns -1 if no token is intersected
    **/
    int get_trump_at_pos(float const, float const );
    int get_trump_at_pos(sf::Vector2f const );
    int get_trump_at_pos(sf::Vector2i const );

    /** \brief what to do when someone clicked **/
    std::shared_ptr<Core::Bet> click(sf::Vector2f const mousePos);
    std::shared_ptr<Core::Bet> click(sf::Vector2i const mousePos);
    std::shared_ptr<Core::Bet> click(float const mouseX, float const mouseY);

private:
    // a general convention is that a non initialized integer is equal to -1.
    std::vector<Token> m_allTokens; // bet tokens, draw only m_nb_tokens of them, but there are the 10 tokens
    AnimatedSprite m_aura_token; // The aura sprite
    AnimatedSprite m_aura_token_selected; // The aura sprite
    std::vector<AnimatedSprite> m_allTrumps; // the trumps vector
    std::vector<AnimatedSprite> m_allTrumps_aura; // the auras linked to the trumps
    std::vector<AnimatedSprite> m_allTrumps_aura_selected; // the auras linked to the trumps
    Core::Bet m_current_bet;
    int m_nbMax;
    int m_mechoune;
    int m_nb_forbidden;
    int m_centerX; // position of the center of the array of trumps, and the tokens are above
    int m_centerY;
    int m_nb_token; // The number of tokens including the black token that are drawn
    int m_trump_clicked; // the current trump clicked
    int m_token_clicked; // the current token clicked
    
    std::vector<Core::Bet> m_betsAvailable;
};

#endif
