#ifndef CONSTANTS
#define CONSTANTS

#include <vector>
#include <string>
#include <exception>

#include "OptionManager.hpp"

/** \file Constants.hpp
*   \brief Definition of all constants of the GUI
**/

// _________ Exception _____________

/** \brief Exception thrown at the end of the game in order to leave the game loop **/
class EndOfGame : public std::exception {
public:
    EndOfGame() = default;
};

// _________ String Trump ___________

extern std::vector<std::string> const NAME_TRUMP;
extern std::vector<std::wstring> const NAME_VALUE;

// ________ Splash Screen ___________

extern double const SPLASH_TIME;
extern double const FADE_SPEED;

// ________ Window Resolutions ________
extern std::vector<std::wstring> const RESOLUTION;

extern std::vector<int> const RESOLUTION_WIDTH;
extern std::vector<int> const RESOLUTION_HEIGHT;

extern int const NB_RESOLUTION;

// ________ Window Size ________
extern int const WINDOW_WIDTH;
extern int const WINDOW_HEIGHT;

extern double const SCALE_RESOLUTION_WIDTH;
extern double const SCALE_RESOLUTION_HEIGHT;
extern double const SCALE_RESOLUTION;

// ______ Colors ______

extern sf::Color const WHITE;
extern sf::Color const YELLOW;
extern sf::Color const BLUE;
extern sf::Color const BLACK;
extern sf::Color const GRAY;
extern sf::Color const GRAY_LIT;
extern sf::Color const GREEN;
extern sf::Color const GREEN_LIT;
extern sf::Color const RED;
extern sf::Color const RED_LIT;
extern sf::Color const TRANSPARENT;
extern sf::Color const BACKGROUND;

// ______ Font size constants ______
extern int const TITLE_FONT_SIZE;
extern int const MENU_BUTTON_FONT_SIZE;
extern int const LOGIN_FONT_SIZE;
extern int const LOGIN_FAIL_FONT_SIZE;
extern int const OPTION_FONT_SIZE;
extern int const OPTION_FONT_WARNING_SIZE;
extern int const STAT_FONT_SIZE;
extern int const TROPHY_NAME_SIZE;
extern int const TROPHY_TEXT_SIZE;

// ______ Math constants ______
extern double const PI;

// _______ Main menu Constants _______
extern int const MENU_SPACE_UNDER_TITLE;
extern int const SELECTION_MENU_SPACE_UNDER_TITLE;
extern int const SELECTION_MENU_SPACE_UNDER_TITLE2;
extern int const SELECTION_MENU_SPACE_UNDER_TITLE3;
extern int const MENU_OFFSET;

//________ Option menu Constants ________
extern int const OPTION_SPACE_UNDER_TITLE;
extern int const OPTION_OFFSET;
extern int const GRAPHIC_OPTION_VERTICAL_LINE_POS;
extern int const GAME_OPTION_VERTICAL_LINE_POS;
extern int const FLAG_SIZE;

//_________ Center Play Zone ________
extern int const CENTER_PLAY_X;
extern int const CENTER_PLAY_Y;

// ______ Player Mark Size _______
extern int const PLAYER_MARK_SIZE;

// ______ Card Size _______
extern int const WIDTH_CARD;
extern int const LENGTH_CARD;

extern double const SCALE_CARD;

extern int const TIME_WAIT_TRICK;
extern int const TIME_TROPHY_APPEAR;

// _______ Bet Bubble Constant _______

extern int const HEIGHT_BETBUBBLE;
extern int const WIDTH_BETBUBBLE;

extern int const SIZE_TEXT_BETBUBBLE;

extern int const DIST_HAND_BETBUBBLE;

extern int const TIME_BETBUBBLE_ANIMATION;

// ________ Text Bubble Constant ________

extern int const FONTSIZE_TUTOBUBBLE;
extern int const OK_BUTTON_SIZE_TUTOBUBBLE;

// _____ Card Speed _______

extern int const C_DISTRIB_TIME;
extern int const C_REPLACE_TIME;
extern int const C_PLAY_TIME;
extern int const C_TRICK_TIME;

// ______ Game Constant ________
extern double const DISTRIBUTION_SPEED;

// ______ Hand Constant ________
extern int const ANGLE_FAN; //degre between each card
extern int const CARD_SPACING; //degree of superposition
extern int const DISTANCE_CIRCLE_CENTER;

// ________ Token Constant _________

extern int const REAL_WIDTH_TOKEN;
extern int const WIDTH_TOKEN;
extern double const SCALE_TOKEN;

// _______ Trophies Drawner Constant ___________
extern int const WIDTH_TROPHIES_DRAWER;
extern int const HEIGHT_TROPHIES_DRAWER;
extern int const SIZE_TROPHIES_PICTURE;

//_________BetArray Constant _________
//Real sizes of variables, the size of trumps' aura is such that it is the same
//for all trumps (and without and all trumps)
extern int const REAL_WIDTH_AURA_TOKEN;
extern int const REAL_HEIGHT_TRUMP;
extern int const REAL_HEIGHT_AURA_TRUMP;
extern int const REAL_WIDTH_ALL_WITHOUT_TRUMP;
extern int const REAL_WIDTH_AURA_TRUMP;

//printed size of trumps
extern int const WIDTH_TRUMP; // = HEIGHT_TRUMP for real trumps

//computation of sizes ingame
extern int const WIDTH_AURA;
extern int const WIDTH_ALL_WITHOUT_TRUMP;
extern int const HEIGHT_AURA_TRUMP;
extern int const WIDTH_AURA_TRUMP;

//distances between elements
extern int const DIST_BETWEEN_TOKEN;
extern int const DIST_BETWEEN_TRUMP;
extern int const DIST_BETWEEN_TOKEN_TRUMP;

#endif
