#ifndef DEF_DROPDOWNMENU
#define DEF_DROPDOWNMENU

#include "Button.hpp"
#include "Core.hpp"

/** \file
*   \author Leo.V, Guillaume.C
*   \brief Definition of class DropDownMenu
**/

enum{PART, TRUMP, TRICK, T_PLAY1, T_PLAY2, T_PLAY3, T_PLAY4, SCORE, S_PLAY1, S_PLAY2, S_PLAY3, S_PLAY4, R_PLAY1, R_PLAY2, R_PLAY3, R_PLAY4, NB_CARDS, NB_BETS, CHOUNE}; //for elements of messageArray
enum{SPR_TRUMP, SPR_PLAY1, SPR_PLAY2, SPR_PLAY3, SPR_PLAY4, CARD, TOKEN}; //for elements of spriteArray


/** \brief The top-left menu of the game screen. Contains various information such has the last trick, the player's score, ... **/
class DropDownMenu {

public:
    /** \brief Constructor.
    *   \param gameState a pointer to the gameState of the current game. Used to update the information available
    **/
    DropDownMenu(const Core::GameState *gameState);

    /** Update function. Call for refresh **/
    void update(sf::RenderWindow& window, sf::Vector2i const mousePos, bool const clicked=false);
    void update(sf::RenderWindow& window, sf::Vector2f const mousePos, bool const clicked=false);
    void update(sf::RenderWindow& window, float const mouseX=-1., float const mouseY=-1., bool const clicked=false);

    /** \brief Movement function. Makes the menu go down **/
    void drop_down();

    /** \brief Movement function. Makes the menu go back up **/
    void fold_back_up();

    /** \brief Movement function. If menu is on top position : drops it down. Otherwise, lifts it up **/
    void react_to_click();

private:
    //_______ DropDownMenu constant _______
    static bool const MENU_ON_RIGHT; //0 false 1 true
    static int const MENU_WIDTH;
    static int const MENU_HEIGHT;
    static int const BUTTON_MENU_SIZE;

    bool m_isDown;

    int const m_dropDownSpeed = 20;

    int m_x; //the position, usefull because heavy expression of x
    int m_y;

    AnimatedSprite m_background;

    int m_numberRound;
    std::vector< AnimatedSprite > m_spriteArray;

    GraphicButton m_upDownButton;
    const Core::GameState *m_gameState;
    std::vector<Message> m_messageArray;


};

#endif
