#ifndef DEF_GRAPHIC_PLAYER
#define DEF_GRAPHIC_PLAYER

/** \file GraphicPlayer.hpp
*   \author Léo.V
*   \brief Definition of class GraphicPlayer
*
**/

#include <SFML/Graphics.hpp>

#include "Hand.hpp"
#include "TokenArray.hpp"
#include "BetBubble.hpp"
#include "PlayerDisplay.hpp"

#include "Core.hpp"

/** \class
*   \brief Contains every element that defines a player in the GUI
*
* Contains a hand of card (class Hand), a tokenArray instance, an icon and a bubble for bets
**/
class GraphicPlayer{
public:
    GraphicPlayer();
    /** \brief Non default constructor
    *   \param
    **/
    GraphicPlayer(sf::Vector2f positionPlayer, int orientationPlayer, double scalePlayer, int mainPlayer);

    void draw(sf::RenderWindow &window);

    void your_turn();
    void end_turn();

    void set_avatar(std::string const &spritename);

    /** \brief trigger event of the left click
            returns a card if trigger a play_card
            returns null if not **/
    std::shared_ptr<Card> handle_mouse_click(float const mouseX, float const mouseY);

    std::shared_ptr<Card> want_play(float const mouseX, float const mouseY);
    std::shared_ptr<Card> want_play(sf::Vector2f const mousePos);
    std::shared_ptr<Card> want_play(sf::Vector2i const mousePos);

    std::shared_ptr<Card> play_card(Core::Card cardPlayed); //play card of an opponent

    void set_play_zone(sf::Vector2f const position);
    void give_card(); //add a card in the hand
	void give_card(Core::Card cardGive);

	void set_playable_cards(Core::Hand);
	void sort_hand(Core::Trump);

	void give_trick();
	void draw_new_trick();
	sf::Vector2f get_pos_free_token();

	void begin_main_phase();
	void end_play_moment();

	void set_bet(Core::Bet bet, bool draw=true);
	void set_message(std::wstring text);
	void set_message(std::string text);
	void restart_bet();
	Core::Bet get_bet();

	int size_hand();

    double get_scale_token();
    bool is_animated();

private:
    Hand m_handPlayer;
    TokenArray m_tokenPlayer;
    BetBubble m_betPlayer;
    PlayerIcon m_iconPlayer;

    bool m_mainPhase;
    bool m_mainPlayer;
    bool m_isActive;
};


#endif
