#ifndef DEF_CLASS_HAND
#define DEF_CLASS_HAND

#include "Card.hpp"

/** \file Hand.hpp
*	\author Léo.V
*	\date 10 October
*	\brief Definition of class Hand
**/

/** \class **/
class Hand{
public:
	Hand() = default;
	Hand(int x, int y, int orientation = 0, double scale = 1, bool mainPlayer = false); //give the position of the center of the hand
	Hand(sf::Vector2f position, int orientation = 0, double scale = 1, bool mainPlayer = false); //give the position of the center of the hand

	void give_card(); //add a card in the hand
	void give_card(Core::Card cardGive);

	void set_play_zone(sf::Vector2f position);
	void set_play_zone(int x, int y);

	void draw(sf::RenderWindow &window);

	int get_card_at_pos(int posX, int posY); // return the card where is the mouse
	int get_card_at_pos(sf::Vector2f const mousePos);
	int get_card_at_pos(sf::Vector2i const mousePos);

	void replace_card(int speed=1); //replace card in the hand
	void sort_hand(Core::Trump);

	void reveal_hand(); //return cards of the hand

	//void new_hand(core) // get the hand in the begin of turn (need core)
	std::shared_ptr<Card> play_card(float const mouseX, float const mouseY);
	std::shared_ptr<Card> play_card(sf::Vector2f const mousePos); // return NULL if they don't play card
	std::shared_ptr<Card> play_card(sf::Vector2i const mousePos);
	//play a random card of the hand and give it a value (for the hand of the opponent)
	std::shared_ptr<Card> play_card(Core::Card typeCard);

	std::shared_ptr<Card> want_play(float const mouseX, float const mouseY);
	std::shared_ptr<Card> want_play(sf::Vector2f const mousePos);
	std::shared_ptr<Card> want_play(sf::Vector2i const mousePos);
	
	void set_playable_cards(Core::Hand);
	
	int size();

private:
	sf::Vector2f compute_position(int i); //call by replace to make fan
	int compute_angle(int i);

	std::vector< std::shared_ptr<Card> > m_handPlayer;
	int m_centerX; //position of the center of the hand
	int m_centerY;
	double m_scaleHand; //the scale of the card

	int m_orientationHand; //orientation of the hand in degree
	Core::Hand m_playableCards;

	bool m_mainPlayer;

	int m_cardMain;

	sf::Vector2f m_playZone; //the position where the card when is played

};

#endif
