#ifndef DEF_PLAYER_DISPLAY
#define DEF_PLAYER_DISPLAY

#include <SFML/Graphics.hpp>
#include <string>
#include "AnimatedSprite.hpp"
#include "Message.hpp"
#include "Constants.hpp"

/** \author Julien.D , GUIllaume.C
*	\file PlayerDisplay.hpp
*	\brief Definition of class PlayerIcon, PlayerDisplay
**/

/** \class **/
class PlayerIcon : public AnimatedSprite {
public:
	PlayerIcon(int size=ICON_SIZE);
    PlayerIcon(std::string const &spriteName, int size=ICON_SIZE);

	void your_turn();
	void end_turn();

	static int const ICON_SIZE;

private:
	bool m_isActive;
};

/** \class **/
class PlayerDisplay {
public:
	PlayerDisplay(float x, float y, int iconSize, int fontSize,
                  std::wstring const &textname,
                  std::string const &fontname,
                  sf::Color const &col = WHITE,
                  sf::Text::Style style = sf::Text::Regular);

    PlayerDisplay(std::string const &spriteName,
		      float x, float y, int iconSize, int fontSize,
		      std::wstring const &textname,
		      std::string const &fontname,
		      sf::Color const &col = WHITE,
		      sf::Text::Style style = sf::Text::Regular);

	void update(sf::RenderWindow &, float const mouseX=-1., float const mouseY=-1.);
	void update(sf::RenderWindow &, sf::Vector2f const);
	void update(sf::RenderWindow &, sf::Vector2i const);

    void your_turn();
    void end_turn();

	/** \brief Setter for the m_name attribute. Do not call TextManager **/
    void set_text(std::wstring const &newText);
	/** \brief Setter for the m_name attribute. Calls TextManager **/
	void set_text(std::string const &key);

    void set_texture(std::string const &spritename);

	void set_font_size(int);
	int get_name_width();

    void set_position(float const imageX, float const imageY, int angle = 0);
    void set_position(sf::Vector2f const positionImage, int angle = 0);
	/** \brief Same as set_position but relative movement **/
	void move(sf::Vector2f const);
	void move(float const, float const);

private:
    PlayerIcon m_icon;
    Message m_name;
	sf::Vector2f m_offset;
};


#endif
