#ifndef DEF_STAT_MANAGER
#define DEF_STAT_MANAGER

#include <SFML/Window.hpp>
#include <SFML/Graphics.hpp>
#include <memory>
#include <iostream>
#include <fstream>

#include "OptionManager.hpp"
#include "enumType.hpp"
#include "Core.hpp"
#include <iostream>
#include <queue>

/** \file
*   \author Léo.V
*   \brief defition of STAT and TROPHY enums, and of StatManager class
**/

enum STAT : int {
    BEST_SCORE,
    BEST_RANK,

    NB_GAME,
    GAME_FIRST,
    GAME_SECOND,
    GAME_THIRD,
    GAME_FOURTH,
    GAME_LEFT,

    NB_ROUNDS,

    TOTAL_POINTS_RECEIVED,
    NB_MECHOUNE_ROUND,
    NB_CHOUNE_ROUND,
    AVERAGE_ROUND_POINTS_RECEIVED,

    NB_TRICKS,
    AVERAGE_ROUND_TRICK,

    NB_TRICK_CAT_CLUB,

    NB_STATS // Size of the enum. /!\ Leave this as the last entry
};

enum class TROPHY : int {
    END_TUTO,
    END_A_GAME,
    MECHOUNE_A_BET,
    CHOUNE_A_BET,
    LOSE_A_GAME,

    WIN_A_GAME,
    WIN_TEN_GAME,
    WIN_HUNDRED_GAME,

    ZERO_POINTS_GAME,
    ZERO_POINTS_ROUND,
    FALL_WITH_ONE_POINT,
    FALL_WITH_TWO_POINTS,
    FALL_WITH_THIRD_POINTS,
    FALL_WITH_VIER_POINTS,
    FALL_WITH_FIVE_POINTS,

    ALL_PLAYERS_FALL,
    TAKE_TRICK_WITH_CAT,
    DOG_ON_CAT, //win a trick with a dog against a cat of the same suit
    LOSE_TRICK_WITH_KING,
    FALL_WHEN_YOUR_MECHOUNE,
    FALL_WHEN_YOUR_CHOUNE,
    DISCOVER_PIRATE_LANGUAGE,

    NB_TROPHIES // Size of the enum. /!\ Leave this as the last entry
};

/** \class
*   \brief Handles everything related to statistics of games, trophies and achievements
**/
class StatManager {
public:
    StatManager();
    ~StatManager();

    /** \brief Reads and loads a profile from a file
    *
    *   Profile files are in the res/profile folder
    **/
    void load_profile(std::string const name);

    /** \brief Writes back the current profile in a file
    *
    *   Profile files are in the res/profile folder
    **/
    void save_profile(std::string const name);

    void begin_game();
    void begin_round(const Core::GameState &gs);

    void end_trick(const Core::GameState &gs);
    void end_round(const Core::GameState &gs);
    void end_game(const Core::GameState &gs);

    void win_trophy(TROPHY trophy);
    bool new_trophy();
    TROPHY get_next_trophy();

    std::vector<int>& get_stats();
    std::vector<int>& get_trophies();

private:
    std::vector<int> m_localStats;
    std::vector<int> m_trophies;
    std::string m_profileName;
    bool m_hasLoaded;
    std::queue<TROPHY> m_bufTrophies;
    int m_bufMechoune;
    int m_bufChoune;
    std::array<int, 4> m_bufScores;
};

extern StatManager statManager; // Global StatManager. Should be the only instance.

#endif
