#ifndef DEF_TUTO_BUBBLE
#define DEF_TUTO_BUBBLE

/** \file
*   \author GUIllaume.C
*   \brief Definition of class TutoBubble
**/

#include <string>
#include <SFML/Graphics.hpp>
#include <SFML/System.hpp>
#include "TutoBubbleSprite.hpp"
#include "Message.hpp"
#include "Button.hpp"

/** \class
*   \brief A bubble used in the tutorials to display messages.
*          Contains a TutoBubbleSprite, which is a set of sprites used to
*          represent the bubble, a message and some buttons
**/
class TutoBubble{
public:
    TutoBubble() = default;

    /** \brief Non default constructor. Calls TextManager in order to retrieve its text
        \param position The position of the top-left corner of the bubble
        \param width is the wanted width of the window. Height is no parameter because it will
                depend on the length of the text and the width
        \param orientation is of enum type PointPosition defined in enumType.hpp
               It tells in which corner the entertainer's face is going to be displayed
    **/
    TutoBubble(sf::Vector2f const position,
               std::string const &key,
               PointPosition const orientation=PointPosition::BOTTOM_RIGHT,
               int width = 300,
               bool okButtonActive = false);

    /** \brief Non default constructor.
        \param position The position of the top-left corner of the bubble
        \param width is the wanted width of the window. Height is no parameter because it will
                depend on the length of the text and the width
        \param orientation is of enum type PointPosition defined in enumType.hpp
    **/
    TutoBubble(sf::Vector2f const position,
                std::wstring const &text,
                PointPosition const orientation= PointPosition::BOTTOM_LEFT,
                int width = 300,
                bool okButtonActive=false);

    /** \brief Update method. Updates the object and display it on the RenderWindow
    *
    *    default position for the mouse is (-1,-1), which is outside the screen
    **/
    void update(sf::RenderWindow& window, float const mouseX=-1., float const mouseY=-1.);
    void update(sf::RenderWindow& window, sf::Vector2f const mousePos);
    void update(sf::RenderWindow& window, sf::Vector2i const mousePos);


    /** \brief Getter method. Says if the bubble is currently moving**/
    bool is_animated();

    /** \brief sets text by calling the TextManager **/
    void set_text(std::string const &key);

    /** \brief sets text directly (no call to TextManager) **/
    void set_text(std::wstring const &text);

    void set_position(sf::Vector2f pos);
    void set_position(sf::Vector2i pos);
    void set_position(float posX, float posY);

    void set_orientation(PointPosition const orient);

    sf::Text get_message_text();
    std::wstring& get_text();

    bool is_ok_active();
    void set_ok_active(bool);

    void show();
    void hide();

    bool button_lit();

private:
    static int const FONTSIZE_TUTOBUBBLE;

    /** \brief Code factoring in different constructors **/
    void init(sf::Vector2f const position,
              std::wstring text,
              PointPosition const orientation,
              int width,
              bool ok_button_active);

    /** \brief Given the number of lines of text in the bubble, computes its total height in pixels **/
    int compute_height(int nb_lines);

    int m_textwidth;
    int m_height;

    PointPosition m_orientation;

    std::wstring m_text;
    sf::Vector2f m_position;

    TutoBubbleSprite m_bubbleSprite;
    GraphicButton m_okButton;

    bool m_okButtonActive;
    bool m_show;

    Message m_message;
};


#endif
