#ifndef DEF_TUTOSPRITEBUBBLE
#define DEF_TUTOSPRITEBUBBLE

/** \file
*   \author GUIllaume.C
*   \brief Definition of class TutoBubbleSprite
**/

#include <string>
#include <SFML/Graphics.hpp>
#include <SFML/System.hpp>
#include "AnimatedSprite.hpp"
#include "Message.hpp"
#include "Button.hpp"
#include "enumType.hpp"

enum class PointPosition {
/**
    Entertainer's position according to the TutoBubble
**/
    TOP_LEFT,
    TOP_RIGHT,
    BOTTOM_LEFT,
    BOTTOM_RIGHT
};

// ________ Text Bubble Constant ________

extern int const OK_BUTTON_SIZE_TUTOBUBBLE;
extern int const LEFT_SPRITE_WIDTH;
extern int const RIGHT_SPRITE_WIDTH;
extern int const LEFT_TEXT_OFFSET;
extern int const TOP_TEXT_OFFSET;
extern int const BOTTOM_TEXT_OFFSET;
extern int const ENTERTAINER_SIZE;
extern int const ENTERTAINER_OFFSET;

extern sf::Vector2f const OK_OFFSET;
extern sf::Vector2f const MESSAGE_OFFSET;
// _______________________________________


class TutoBubble;

/** \class
*   \brief A set of sprites representing a bubble of customizable size.
           Mainly used in TutoBubble class, which is used in the tutorial section
**/
class TutoBubbleSprite{

friend class TutoBubble;
/* This class represents some factoring of various objects. Its only use is to be an attribute of the TutoBubble class.
Therefore, TutoBubble is a friend class, and TutoBubbleSprite's constructor is private. */

public:
    TutoBubbleSprite() = default;

private:

    /** \brief Non default constructor.
        \param position The position of the top-left corner of the bubble
        \param width is the wanted width of the window. Height is no parameter because it will
                depend on the length of the text and the width
        \param orientation is of enum type PointPosition defined in enumType.hpp
               It tells in which corner the entertainer's face is going to be displayed
    **/
    TutoBubbleSprite(sf::Vector2f const position,
               PointPosition const orientation=PointPosition::BOTTOM_RIGHT,
               int width = 300,
               int height = 200);



    void set_position(float posX, float posY);

    void update(sf::RenderWindow& window);

    /** \brief Getter method. Says if the bubble is currently moving**/
    bool is_animated();

    sf::Vector2f get_dimensions();

    double m_width;
    double m_height;

    sf::Vector2f m_origin;
    sf::Vector2f m_centerBubble;
    sf::Vector2f m_midOffset;
    sf::Vector2f m_rightOffset;
    sf::Vector2f m_entertainerOffset;

    AnimatedSprite m_spriteBubbleLeft;
    AnimatedSprite m_spriteBubbleRight;
    AnimatedSprite m_spriteBubbleMiddle;
    AnimatedSprite m_spriteEntertainer;
};

#endif
