#ifndef DEF_TUTO_GAME
#define DEF_TUTO_GAME

#include "GraphicGame.hpp"
#include "TutoBubble.hpp"
#include <fstream>
#include <queue>
#include <thread>
#include <mutex>
#include <condition_variable>

// ____________________________________

/**  \file
*    \author Léo.V, Louis.B, GUIllaume.C
*    \brief Definition of class TutoGraphicGame
*
*    Documentation by GUIllaume.C
**/


struct Sleeper {
    public:
        Sleeper(std::mutex&, std::condition_variable&, bool&);
        ~Sleeper();
    private:
        std::unique_lock<std::mutex> lock;
        std::condition_variable * const cv;
        bool * const waiting;
};

class TutoKernel {
public:
    /**  \brief Constructor **/
    TutoKernel(std::string name, Core::GameState* gamestate, Core::ParallelPlayer* player, GraphicGame* graphicGame);

    void run();

    bool read_instruction();

    void read_bet();
    void read_play();
    void read_multiplay();
    void read_message();
    void read_mechoune();
    void read_choune();
    void read_hand();
    void read_setbet();
    void read_comment();
    void read_scores();
    void read_trick();

    bool tuto_ending();
    void close();

    void getMessage(std::string &messageKey, bool &newMessage, bool &pause, sf::Vector2f &position);
    std::string messageKey() const;
    bool new_message();
    bool is_pause() const;

    // ________ Mutex Button ______________

    static std::condition_variable okVariable;
    static std::mutex okMutex;
    static bool okReady;

    static std::mutex messageMutex;

private:
    std::string m_nameFile;
    std::ifstream m_file;
    Core::GameState* m_gamestate;
    Core::ParallelPlayer* m_player;
    GraphicGame* m_graphicGame;

    std::string m_messageKey;
    sf::Vector2f m_posMessage;
    bool m_newMessage;
    bool m_pause;
    bool m_tutoEnding;
    bool m_close;
};

void run_tuto(std::shared_ptr<TutoKernel> tuto);


#endif
