#pragma once

#include <memory>

#include "Core.hpp"
#include "KnowledgeEngine.hpp"

/** \file
*   \author Nicolas, Robin, Tristan
*   \brief Definition of class IA::BotDeter
*   Implement the pseudo code on https://pad.aliens-lyon.fr/p/heuristiques
**/

namespace IA {

    bool overall_aggressive_game(const std::array<int, 4ul>&,const std::array<int, 4ul>&, int);

    bool deeply_defensive_game(const Core::GameState&);

    bool may_be_in_game(const Core::Card, const KnowledgeEngine&);

    struct BotDeterParameters {
        BotDeterParameters(const Core::GameState&, const KnowledgeEngine&);
        const Core::GameState& gs;
        const KnowledgeEngine& ke;

        Core::Hand winning_cards;
        Core::Hand my_trumps;

        bool aggressive;
        bool deeply_defensive;
        bool overpowered_hand;
        bool can_win;

        bool first_player;
        bool middle_player;

        bool aiming_min;
        bool aiming_max;

        bool doit_defausser;
        bool doit_couper;
        bool doit_fournir_atout;
        bool doit_monter_atout;
        bool est_coupe;

        float q;
    };

    class BotDeter final : public Core::Player {
    public:
        BotDeter();

        Core::Bet bet();
        bool mechoune();
        bool choune();
        Core::Card play();

        virtual void ready();
        virtual void set_game_state(const Core::GameState& gameState);

    private:
        /*
        * Here you can add the methods you need for each case
        */
        Core::Card play_NoTrump(const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);
        Core::Card play_NoTrump_aiming_min(const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);
        Core::Card play_NoTrump_aiming_max(const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);
        Core::Card play_NoTrump_in_between(const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);

        Core::Card play_AllTrump(const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);
        Core::Card play_AllTrump_aiming_min(const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);
        Core::Card play_AllTrump_aiming_max(const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);
        Core::Card play_AllTrump_in_between(const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);

        Core::Card play_OneTrump(const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);
        Core::Card play_OneTrump_aiming_min(const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);
        Core::Card play_OneTrump_aiming_max(const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);
        Core::Card play_OneTrump_in_between(const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);

        Core::Card pick_strongest(const Core::Hand);
        Core::Card pick_strongest_weight(const Core::Hand, const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);
        Core::Card pick_weekest(const Core::Hand);
        Core::Card pick_weekest_weight(const Core::Hand, const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);

        float winning_probability(const Core::Card, const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);
        Core::Hand m_good_winning_proba(const Core::Hand, const float, const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);
        Core::Hand m_good_losing_proba(const Core::Hand, const float, const Core::GameState&, const KnowledgeEngine&, const BotDeterParameters&);


        void processEvent();
        std::unique_ptr<KnowledgeEngine> m_engine;
    };
}
