#ifndef IA_PLAYER
#define IA_PLAYER

#include <memory>
#include "Core.hpp"
#include "MonteCarlo.hpp"

/** \file
*   \author Léo.V and Louis.B
*   \brief Definition of class IA::IAPlayer
**/

namespace IA {
    /** Class IAPlayer inherits from class Player
    * \brief A IA which used intelligent technic like Monte-Carlo
    **/
    struct Loss {
        Loss(Core::Trump, int, int);
        Core::Bet bet;
        int totalError;
    };
    bool operator<(const Loss&, const Loss&);

    class IAPlayer final: public Core::Player {
    public:
        IAPlayer(int, std::array<double, 10> coefError = {1.,0.9, 0.75, 0.6, 0.4, 0.2, 0., 0., 0., 0.});

        Core::Bet bet();
        bool mechoune();
        bool choune();
        Core::Card play();

        void ready();

        virtual void set_game_state(const Core::GameState& gameState);

    private:
        void processEvent();
        void compute_loss(const std::array<std::vector<int>, 6>&);
        Core::Bet choose_bet(const std::vector<Loss>&) const;
        void precompute();
        void print_mechoune();

        std::unique_ptr<KnowledgeEngine> m_engine;
        std::vector<Loss> m_loss;
        
        int m_nbExplorations;
        std::array<double, 10> m_errorCoeff;
        
    };
}

#endif
