#pragma once

#include "GameState.hpp"

/** \file Player.hpp
*   \brief Definition of class Core::Player
**/

namespace Core {
    /** \brief A player that can do action to change the state of the game **/
    class Player {
    public:
        virtual ~Player() {}
        /** \brief The bet of a player **/
        virtual Bet bet() = 0;
        /** \brief The fact that a player is 'mechouning'  **/
        virtual bool mechoune() = 0;
        /** \brief The fact that a player is 'chouning'  **/
        virtual bool choune() = 0;
        /** \brief The fact that a player play **/
        virtual Card play() = 0;
        /** \brief The fact that a player is ready **/
        virtual void ready() {};

        virtual int my_id() const final {
            return m_gameState->my_id();
        }

        /** \brief  Set the state of the game
        *   \param gameState The state of a game for the player
        **/
        virtual void set_game_state(const GameState& gameState){
            m_gameState = &gameState;
        }
        /** \brief  Return the state of the game for the player
        *   \return An immutable state of a game
        **/
        virtual const GameState& get_game_state() final {
            return *m_gameState;
        }

        virtual std::unique_ptr<GameState> create_game_state(int id) {
            return std::unique_ptr<GameState>(new GameState(id));
        }

    protected:
         /** \brief The GameState of the player **/
        GameState const * m_gameState;
    };
}
