#pragma once

#include <SFML/Network.hpp>
#include <string>
#include <memory>
#include <sodium.h>

#include "ServerConstants.hpp"
#include "CryptoBox.hpp"

using namespace std;

namespace Network {

    /** \brief An enum to represent types of message with known values for interfacing **/
    enum class ServerHeader: char
        {
            ReturnStatus=0,
            
            CreateGame=1,
            JoinGame=2,
            FillGame=3,

            StartGame=4
        };

    /** \brief An object that contains and handle a buffer for in-game communications.
        A GameMessage handles a char buffer of size tcpServerBufferSize, and has methods to fill it with the various messages needed during the game and to read the received messages. **/
    class ServerMessage {
    public:
        ServerMessage();

        ServerHeader header() const;
        unsigned char * data();

		void receive(shared_ptr<sf::TcpSocket>, CryptoBox *);
    	void send(shared_ptr<sf::TcpSocket>, CryptoBox *);

        void write_header(ServerHeader);
        void write_lobby(char);
        void write_status(Status);

        char read_lobby() const;
        Status read_status() const;
        
    private:
       size_t m_received;
       unsigned char m_data[dataSize];
    };

	Status translate_status(sf::Socket::Status);
}
