#include "Background.hpp"

Background::Background(){
    m_alpha = 255;
    m_goal_alpha = 255;
    m_speed = 0;
}

Background::Background(std::string const &name) :
    Background::Background()
{
    set_texture(name);
}

void Background::set_texture(std::string const &name) {
    m_sprite.setTexture(RM.get_texture(name));
    sf::Vector2u size = m_sprite.getTexture()->getSize();
    m_sprite.setScale( float(WINDOW_WIDTH) / size.x ,  float(WINDOW_HEIGHT) / size.y);
}

void Background::set_transparency(int alpha){
    m_alpha = alpha;
    m_goal_alpha = alpha;
    m_sprite.setColor(sf::Color(255,255,255,m_alpha));
}

void Background::fade(int alpha, double speed) {
    m_goal_alpha = alpha;
    m_speed = speed;
}

void Background::continue_fade(){
    int delta = m_goal_alpha - m_alpha;
    if (abs(delta) > m_speed) {
        double scaling = double(delta) / m_speed;
        m_alpha += scaling;
    } else if (delta!=0) {
        m_alpha = m_goal_alpha;
    }
    m_sprite.setColor(sf::Color(255,255,255,m_alpha));
}

bool Background::is_fading() const {
    return (m_alpha!=m_goal_alpha);
}

void Background::draw(sf::RenderWindow& bliton) const {
    if (m_alpha>0){
        bliton.draw(m_sprite);
    }
}

void Background::update(sf::RenderWindow& bliton){
    continue_fade();
    if (m_alpha>0) {
        bliton.draw(m_sprite);
    }
}
