#include "BetBubble.hpp"
#include "geometry.hpp"
#include "Constants.hpp"
#include "TextManager.hpp"

BetBubble::BetBubble(sf::Vector2f positionBubble, int idPlayer, int angleBubble, double scaleBubble) :
    m_bet(Core::Trump::NoTrump, 0),
    m_spriteBubble(WIDTH_BETBUBBLE,HEIGHT_BETBUBBLE),
    m_iconTrump(SIZE_TEXT_BETBUBBLE,SIZE_TEXT_BETBUBBLE),
    m_messageBubble(positionBubble.x, positionBubble.y,SIZE_TEXT_BETBUBBLE,L"","MPlantin_0.ttf",BLACK, Align::CENTER)
{
    m_positionBubble = positionBubble;
    m_centerBubble = positionBubble - sf::Vector2f(0,30*SCALE_CARD) - sf::Vector2f(0, 25*(1-idPlayer%2)*(1-idPlayer)*SCALE_CARD);
    m_startPosition = m_positionBubble + rotateVector(0, HEIGHT_BETBUBBLE/2, angleBubble);
    switch(idPlayer){
    case 0:
        m_spriteBubble.set_texture("bubbles/bulle_pointe_ellipse.png");
        break;
    case 1:
        m_spriteBubble.set_texture("bubbles/bulle_senestre_ellipse.png");
        break;
    case 2:
        m_spriteBubble.set_texture("bubbles/bulle_chef_ellipse.png");
        break;
    case 3:
        m_spriteBubble.set_texture("bubbles/bulle_dextre_ellipse.png");
        break;
    }
    m_spriteBubble.set_position(m_startPosition);
    m_iconTrump.set_position(m_startPosition);
    m_messageBubble.set_position(m_startPosition);
    m_spriteBubble.set_scale(0);
    m_iconTrump.set_scale(0);
    m_messageBubble.set_scale(0);
    m_angleBubble = angleBubble;
    m_onlyText = true;
    m_draw = false;
}

void BetBubble::set_bet(Core::Bet bet, bool draw){
    m_bet = bet;
    if(!draw)
        return;
    if(m_bet.trump == Core::Trump::NoTrump){
        m_iconTrump.set_texture("sprites_token/sansat-noir.png");
    }
    else if(m_bet.trump == Core::Trump::AllTrump){
        m_iconTrump.set_texture("sprites_token/toutat-noir.png");
    }
    else{
        m_iconTrump.set_texture("sprites_token/"+trump_to_string(m_bet.trump)+".png");
    }
    m_messageBubble.set_text(std::to_wstring(m_bet.value));
    m_onlyText = false;
    m_draw = true;

    animate();
}

void BetBubble::set_message(std::wstring const &text){
    m_onlyText = true;
    m_draw = true;
    m_messageBubble.set_text(L"     "+text+L"!"); // Attention, il faut une espace en français et pas en englais devant un '!'.
    animate();
}

void BetBubble::set_message(std::string const &textname){
    set_message(textManager.get_entry(textname));
}

void BetBubble::update(sf::RenderWindow& window){
    m_spriteBubble.continue_move();
    if(m_draw){
        window.draw(m_spriteBubble);
        m_messageBubble.update(window);
        if(!m_onlyText){
            m_iconTrump.update(window);
        }
    }
}

bool BetBubble::is_animated(){
    return m_spriteBubble.is_animated();
}

void BetBubble::animate(){
    m_messageBubble.set_position(m_startPosition);
    m_spriteBubble.set_position(m_startPosition);
    m_iconTrump.set_position(m_startPosition);
    m_messageBubble.set_scale(0);
    m_spriteBubble.set_scale(0);
    m_iconTrump.set_scale(0);

    m_messageBubble.move_sprite_in_time(m_centerBubble + sf::Vector2f(-30*SCALE_CARD,15*SCALE_CARD), 0, TIME_BETBUBBLE_ANIMATION);
    m_spriteBubble.move_sprite_in_time(m_positionBubble, 0, TIME_BETBUBBLE_ANIMATION);
    m_iconTrump.move_sprite_in_time(m_centerBubble+sf::Vector2f(30*SCALE_CARD,30*SCALE_CARD), 0, TIME_BETBUBBLE_ANIMATION);
    m_messageBubble.zoom_sprite(1, TIME_BETBUBBLE_ANIMATION);
    m_spriteBubble.zoom_sprite(1, TIME_BETBUBBLE_ANIMATION);
    m_iconTrump.zoom_sprite(1, TIME_BETBUBBLE_ANIMATION);
}

Core::Bet BetBubble::get_bet(){
    return m_bet;
}

void BetBubble::restart(){
    sf::Vector2f move( rotateVector(0, -HEIGHT_BETBUBBLE/2, m_angleBubble) );

    m_messageBubble.move_sprite_in_time(m_startPosition, 0, TIME_BETBUBBLE_ANIMATION);
    m_spriteBubble.move_sprite_in_time(m_startPosition, 0, TIME_BETBUBBLE_ANIMATION);
    m_iconTrump.move_sprite_in_time(m_startPosition, 0, TIME_BETBUBBLE_ANIMATION);
    m_messageBubble.zoom_sprite(0, TIME_BETBUBBLE_ANIMATION);
    m_spriteBubble.zoom_sprite(0, TIME_BETBUBBLE_ANIMATION);
    m_iconTrump.zoom_sprite(0, TIME_BETBUBBLE_ANIMATION);
}
