#include "TutoBubble.hpp"
#include "TextManager.hpp"
#include "geometry.hpp"
#include "Constants.hpp"
#include "stringSplit.hpp"

int const TutoBubble::FONTSIZE_TUTOBUBBLE=25;

TutoBubble::TutoBubble(sf::Vector2f const position,
           std::string const &key,
           PointPosition const orientation,
           int width,
           bool okButtonActive) :
    TutoBubble(position, textManager.get_entry(key), orientation, width, okButtonActive) {}

TutoBubble::TutoBubble(sf::Vector2f const position,
           std::wstring const &text,
           PointPosition const orientation,
           int width,
           bool okButtonActive)
{
    m_show = true;
    m_text = text;
    m_textwidth=width;
    m_okButtonActive = okButtonActive;
    m_position = position;

    int nb_lines = split_string(m_text, FONTSIZE_TUTOBUBBLE, m_textwidth);
    m_height = compute_height(nb_lines);

    m_bubbleSprite = TutoBubbleSprite(position, orientation, width, m_height);

    m_okButton = GraphicButton(position.x + width - OK_BUTTON_SIZE_TUTOBUBBLE, position.y + m_height - OK_BUTTON_SIZE_TUTOBUBBLE- OK_OFFSET.y/3, "ok_tuto.png", "ok_tuto_lit.png", "ok_tuto_lit.png", OK_BUTTON_SIZE_TUTOBUBBLE, OK_BUTTON_SIZE_TUTOBUBBLE, false);
    m_message = Message(position.x, position.y, FONTSIZE_TUTOBUBBLE, m_text ,"MPlantin_0.ttf", BLACK, Align::LEFT);

    m_message.set_position(position + MESSAGE_OFFSET + sf::Vector2f(0,m_height/2));
}

int TutoBubble::compute_height(int nb_lines){
    return TOP_TEXT_OFFSET+nb_lines*(FONTSIZE_TUTOBUBBLE+10) + (m_okButtonActive ? OK_BUTTON_SIZE_TUTOBUBBLE+OK_OFFSET.y : BOTTOM_TEXT_OFFSET);
}

void TutoBubble::set_text(std::string const &key){
    set_text(textManager.get_entry(key));
}

void TutoBubble::set_text(std::wstring const &text){
    m_text = text;
    int nb_lines = split_string(m_text, FONTSIZE_TUTOBUBBLE, m_textwidth);
    m_height = compute_height(nb_lines);
    m_bubbleSprite = TutoBubbleSprite(m_position, m_orientation, m_textwidth, m_height);
    m_okButton.set_position(m_position.x + m_textwidth - OK_BUTTON_SIZE_TUTOBUBBLE, m_position.y + m_height - OK_BUTTON_SIZE_TUTOBUBBLE- OK_OFFSET.y/3);
    m_message.set_text(m_text);
    m_message.set_position(m_position.x+LEFT_TEXT_OFFSET, m_position.y+TOP_TEXT_OFFSET+m_height/2);
}

void TutoBubble::set_position(sf::Vector2f pos) {
    set_position(pos.x, pos.y);
}

void TutoBubble::set_position(sf::Vector2i pos) {
    set_position(pos.x, pos.y);
}

void TutoBubble::set_position(float posX, float posY) {
    m_message.set_position(posX+LEFT_TEXT_OFFSET,posY+TOP_TEXT_OFFSET+m_height/2);
    m_bubbleSprite.set_position(posX,posY);
    m_okButton.set_position(posX + m_textwidth - OK_BUTTON_SIZE_TUTOBUBBLE, posY + m_height - OK_BUTTON_SIZE_TUTOBUBBLE- OK_OFFSET.y/3);

}

void TutoBubble::set_orientation(PointPosition const orient) {
    m_bubbleSprite = TutoBubbleSprite(m_position, orient, m_textwidth, m_height);
}

sf::Text TutoBubble::get_message_text(){
    return m_message.get_text();
}

std::wstring& TutoBubble::get_text(){
    return m_text;
}

void TutoBubble::update(sf::RenderWindow& window, sf::Vector2i const mousePos){
    update(window, mousePos.x, mousePos.y);
}

void TutoBubble::update(sf::RenderWindow& window, sf::Vector2f const mousePos){
    update(window, mousePos.x, mousePos.y);
}

void TutoBubble::update(sf::RenderWindow &window, float const mouseX, float const mouseY) {
    if (m_show){
        m_bubbleSprite.update(window);
        m_message.update(window);
        if (m_okButtonActive) {
            m_okButton.update(window, mouseX, mouseY);
        }
    }
}

bool TutoBubble::is_animated(){
    return m_bubbleSprite.is_animated();
}

bool TutoBubble::is_ok_active(){
    return m_okButtonActive;
}

void TutoBubble::set_ok_active(bool b){
    m_okButtonActive=b;
}

bool TutoBubble::button_lit(){
    return m_okButton.is_lit();
}

void TutoBubble::show(){
    m_show=true;
}

void TutoBubble::hide(){
    m_show=false;
}
