#include "TutoBubbleSprite.hpp"
#include "RessourceManager.hpp"
#include "geometry.hpp"
#include "Constants.hpp"

// ________ Tuto Bubble Constants ________

int const OK_BUTTON_SIZE_TUTOBUBBLE = 30;
int const LEFT_SPRITE_WIDTH = 50;
int const RIGHT_SPRITE_WIDTH = 100;
int const LEFT_TEXT_OFFSET = 30;
int const TOP_TEXT_OFFSET = -10;
int const BOTTOM_TEXT_OFFSET = 30;
int const ENTERTAINER_SIZE = 80;
int const ENTERTAINER_OFFSET = 70;

sf::Vector2f const OK_OFFSET(50,50);
sf::Vector2f const MESSAGE_OFFSET(LEFT_TEXT_OFFSET, TOP_TEXT_OFFSET);

// _______________________________________

TutoBubbleSprite::TutoBubbleSprite(sf::Vector2f const position,
                                   PointPosition const orientation,
                                   int width,
                                   int height)
{
    m_width = width;
    m_height = height;

    m_spriteBubbleLeft = AnimatedSprite(LEFT_SPRITE_WIDTH, m_height, false);
    m_spriteBubbleRight = AnimatedSprite(RIGHT_SPRITE_WIDTH, m_height, false);

    int middleSpriteWidth = m_width-(LEFT_SPRITE_WIDTH+RIGHT_SPRITE_WIDTH)/2;
    m_spriteBubbleMiddle = AnimatedSprite(middleSpriteWidth, m_height, false);

    m_spriteEntertainer = AnimatedSprite(ENTERTAINER_SIZE, ENTERTAINER_SIZE, false);
    m_spriteEntertainer.set_texture("fou.png");

    m_origin = position;
    m_centerBubble = position + sf::Vector2f(m_width/2, m_height/2);

    m_midOffset = sf::Vector2f(LEFT_SPRITE_WIDTH,0);
    m_rightOffset = sf::Vector2f(LEFT_SPRITE_WIDTH+middleSpriteWidth,0);
    m_entertainerOffset = sf::Vector2f(m_width+ENTERTAINER_OFFSET, m_height/2);

    switch(orientation){
       case PointPosition::TOP_LEFT:
           m_spriteBubbleMiddle.set_texture("bubbles/fou_milieu.png");
           m_spriteBubbleLeft.set_texture("bubbles/fou_gauche.png");
           m_spriteBubbleRight.set_texture("bubbles/fou_droite.png");
           break;
       case PointPosition::TOP_RIGHT:
           m_spriteBubbleMiddle.set_texture("bubbles/fou_milieu.png");
           m_spriteBubbleLeft.set_texture("bubbles/fou_gauche.png");
           m_spriteBubbleRight.set_texture("bubbles/fou_droite.png");
           break;
       case PointPosition::BOTTOM_LEFT:
           m_spriteBubbleMiddle.set_texture("bubbles/fou_milieu.png");
           m_spriteBubbleLeft.set_texture("bubbles/fou_gauche.png");
           m_spriteBubbleRight.set_texture("bubbles/fou_droite.png");
           break;
       case PointPosition::BOTTOM_RIGHT:
       default:
           m_spriteBubbleMiddle.set_texture("bubbles/fou_milieu.png");
           m_spriteBubbleLeft.set_texture("bubbles/fou_gauche.png");
           m_spriteBubbleRight.set_texture("bubbles/fou_droite.png");
           break;
    }
    m_spriteBubbleLeft.set_position(m_origin);
    m_spriteBubbleMiddle.set_position(m_origin + m_midOffset);
    m_spriteBubbleRight.set_position(m_origin + m_rightOffset);
    m_spriteEntertainer.set_position(m_origin + m_entertainerOffset);
}

sf::Vector2f TutoBubbleSprite::get_dimensions(){
    return sf::Vector2f(m_width,m_height);
}

void TutoBubbleSprite::set_position(float posX, float posY){
    sf::Vector2f pos(posX,posY);
    m_spriteBubbleLeft.set_position(pos);
    m_spriteBubbleMiddle.set_position(pos + m_midOffset);
    m_spriteBubbleRight.set_position(pos + m_rightOffset);
    m_spriteEntertainer.set_position(pos + m_entertainerOffset);
}

void TutoBubbleSprite::update(sf::RenderWindow &window) {
    m_spriteBubbleLeft.update(window);
    m_spriteBubbleRight.update(window);
    m_spriteBubbleMiddle.update(window);
    m_spriteEntertainer.update(window);
}

bool TutoBubbleSprite::is_animated(){
    return m_spriteBubbleMiddle.is_animated();
}
