#include "Button.hpp"

using namespace sf;

GraphicButton::GraphicButton(float pos_x, float pos_y, std::string const &textureBasePath, std::string const &textureLitPath, std::string const &textureClickedPath, int width, int height, bool originAtCenter) :
  Button(),
  m_buttonSprite(width, height, originAtCenter)
{

    m_buttonBaseTexture = textureBasePath;
    m_buttonLitTexture = textureLitPath;
    m_buttonClickedTexture = textureClickedPath;
    m_buttonSprite.set_texture(textureBasePath);
    m_buttonSprite.set_position(pos_x, pos_y);
}

void GraphicButton::lock() {
    m_isSelected = true;
    m_buttonSprite.set_texture(RM.get_texture(m_buttonClickedTexture));
}

void GraphicButton::update(sf::RenderWindow &window, float const mouse_x, float const mouse_y){
    if(not is_selected()) {
        if((m_buttonSprite.pos_on_sprite(mouse_x, mouse_y)^is_lit())) {
            if (is_lit()) {
                m_buttonSprite.set_texture(RM.get_texture(m_buttonBaseTexture));
            }
            else {
                m_buttonSprite.set_texture(RM.get_texture(m_buttonLitTexture));
            }
            m_isLit = !m_isLit;
        }
    }
    m_buttonSprite.update(window);
}

void GraphicButton::update(sf::RenderWindow &window, sf::Vector2f const mousePos) {
    update(window, mousePos.x, mousePos.y);
}

void GraphicButton::update(sf::RenderWindow &window, sf::Vector2i const mousePos) {
    update(window, mousePos.x, mousePos.y);
}

// ______ Movement of sprites ______

void GraphicButton::set_position(float const posX, float const posY) {
    m_buttonSprite.set_position(posX, posY);
}

void GraphicButton::set_position(sf::Vector2f const pos) {
    set_position(pos.x, pos.y);
}

void GraphicButton::set_position(sf::Vector2i const pos) {
    set_position(pos.x, pos.y);
}

void GraphicButton::move_sprite(sf::Vector2i position, int angle, int speed){
	m_buttonSprite.move_sprite(Vector2f(position), angle, speed);
}

void GraphicButton::move_sprite_relative(int x, int y, int angle, int speed){
    m_buttonSprite.move_sprite(x+m_buttonSprite.getPosition().x,y+m_buttonSprite.getPosition().y,angle,speed);
}

void GraphicButton::move_sprite_relative(sf::Vector2f position, int angle, int speed){
    m_buttonSprite.move_sprite(position+m_buttonSprite.getPosition(), angle, speed);
}

void GraphicButton::move_sprite_relative(sf::Vector2i position, int angle, int speed){
	m_buttonSprite.move_sprite(sf::Vector2f(position)+m_buttonSprite.getPosition(), angle, speed);
}

void GraphicButton::set_angle(int angle){
    m_buttonSprite.set_angle(angle);
}
