#include "Button.hpp"
#include "SoundManager.hpp"

LinkedButtons::LinkedButtons(int n, std::vector<float> pos_x, std::vector<float> pos_y, std::vector<int> size, std::vector<std::string> &text, std::vector<std::string> &fontname, std::vector<sf::Color> &col, std::vector<sf::Color> &litCol,std::vector<sf::Color> &clickedCol, sf::Text::Style style, Align align, int selected) {
    m_size=n;
    for(int i=0;i<n;i++) {
        TextButton tmp(pos_x[i], pos_y[i], size[i], text[i], fontname[i], col[i], litCol[i], clickedCol[i], align, style);
        m_textButtons.push_back(tmp);
    }
    m_textButtons[selected].lock();
    m_textButtons[selected].force_lit();
    m_selected = selected;
    m_isGraphic = false;
}

LinkedButtons::LinkedButtons(int n, std::vector<float> pos_x, std::vector<float> pos_y, std::vector<std::string> &textureBasePath, std::vector<std::string> &textureLitPath, std::vector<std::string> &textureClickedPath, std::vector<int> width, std::vector<int> height, bool originAtCenter, int selected) {
    m_size=n;
    for(int i=0;i<n;i++) {
        GraphicButton tmp(pos_x[i], pos_y[i], textureBasePath[i], textureLitPath[i], textureClickedPath[i], width[i], height[i], originAtCenter);
        m_graphicButtons.push_back(tmp);
    }
    m_graphicButtons[selected].lock();
    m_graphicButtons[selected].force_lit();
    m_selected = selected;
    m_isGraphic = true;
}

LinkedButtons::LinkedButtons(std::vector<TextButton> &buttons, int selected) {
  m_size = buttons.size();
  m_isGraphic = false;
  m_textButtons = buttons;
  m_textButtons[selected].lock();
  m_textButtons[selected].force_lit();
  m_selected=selected;
}

LinkedButtons::LinkedButtons(std::vector<GraphicButton> &buttons, int selected) {
  m_size = buttons.size();
  m_isGraphic = true;
  m_graphicButtons = buttons;
  m_graphicButtons[selected].lock();
  m_graphicButtons[selected].force_lit();
  m_selected=selected;
}

void LinkedButtons::change_selected(int new_selected) {
    if(m_isGraphic) {
        m_graphicButtons[m_selected].unlock();
        m_graphicButtons[new_selected].lock();
        m_graphicButtons[m_selected].force_lit();
    }
    else {
        m_textButtons[m_selected].unlock();
        m_textButtons[new_selected].lock();
        m_textButtons[m_selected].force_lit();
    }
    m_selected = new_selected;
}

int LinkedButtons::who_is_selected() {
    return m_selected;
}

void LinkedButtons::update(sf::RenderWindow &app, sf::Vector2f const mousePos, bool clicked) {
    update(app, mousePos.x, mousePos.y, clicked);
}

void LinkedButtons::update(sf::RenderWindow &app, sf::Vector2i const mousePos, bool clicked) {
    update(app, mousePos.x, mousePos.y, clicked);
}

void LinkedButtons::update(sf::RenderWindow &app, float const mouseX, float const mouseY, bool clicked) {
    int ancSelected=who_is_selected();
    if(m_isGraphic) {
        for(int i=0;i<m_size;i++) {
            m_graphicButtons[i].update(app, mouseX, mouseY);
            if(m_graphicButtons[i].is_lit() && clicked) {
                soundManager.play_sound("click.wav");
    	        change_selected(i);
    	        m_graphicButtons[ancSelected].update(app, mouseX, mouseY);
    	        m_graphicButtons[i].update(app, mouseX, mouseY);
            }
        }
    }
    else {
        for(int i=0;i<m_size;i++) {
            m_textButtons[i].update(app, mouseX, mouseY);
            if(m_textButtons[i].is_lit() && clicked) {
                soundManager.play_sound("click.wav");
    	        change_selected(i);
    	        m_textButtons[ancSelected].update(app, mouseX, mouseY);
    	        m_textButtons[i].update(app, mouseX, mouseY);
    	    }
    	}
    }
}
