#include "SliderButton.hpp"
#include "SoundManager.hpp"
#include "Constants.hpp"

using namespace sf;

SliderButton::SliderButton(float pos_x, float pos_y,
			   int disk_size,
			   float length,
			   sf::Color color,
			   std::string const &fontname,
			   float base_value)
{
    m_value = base_value;
	m_color = color;
	m_litColor = GRAY;
    m_circle = sf::CircleShape(disk_size);
    m_line[0] = sf::Vertex(sf::Vector2f(pos_x, pos_y));
    m_line[1] = sf::Vertex(sf::Vector2f(pos_x+length, pos_y));
    m_moving = false;
    m_circle.setPosition(pos_x+base_value*length-disk_size, pos_y-disk_size);
    m_circle.setFillColor(m_color);
    m_length = length;
    m_radius = disk_size;
    m_x = pos_x;
    m_y = pos_y;
    m_text = Message(pos_x+length+3*disk_size, pos_y-disk_size, 2*disk_size, std::to_wstring((int) (base_value*100)), fontname, color);
}

float SliderButton::get_value() {
    return m_value;
}

void SliderButton::update(sf::RenderWindow &app, bool mouseLeftClick, float const mouseX, float const mouseY) {
    if(not m_moving and m_circle.getGlobalBounds().contains(mouseX, mouseY) and mouseLeftClick) {
		soundManager.play_sound("click.wav");
        m_moving = true;
		m_circle.setFillColor(m_litColor);
    }
    else if(m_moving and not mouseLeftClick) {
		soundManager.play_sound("click.wav");
        m_moving = false;
		m_circle.setFillColor(m_color);
    }
    if(m_moving) {
        if (mouseX>=m_x and mouseX<=m_x+m_length) {
	    m_value = (mouseX-m_x)/m_length;
	    m_circle.setPosition(mouseX-m_radius, m_y-m_radius);
	    m_text.set_text(std::to_wstring( (int) (m_value*100)));
	}
	else if (mouseX<m_x and m_value !=0) {
	    m_value = 0;
	    m_circle.setPosition(m_x-m_radius, m_y-m_radius);
	    m_text.set_text(std::to_wstring(0));
	}
	else if (mouseX>m_x+m_length and m_value != 1) {
	    m_value = 1;
	    m_circle.setPosition(m_x+m_length-m_radius, m_y-m_radius);
	    m_text.set_text(std::to_wstring(100));
	}
    }
    m_text.update(app);
    app.draw(m_circle);
    app.draw(m_line, 2, sf::Lines);

}

void SliderButton::update(sf::RenderWindow &app, bool mouseLeftClick, sf::Vector2f mousePos) {
    update(app, mouseLeftClick, mousePos.x, mousePos.y);
}

void SliderButton::update(sf::RenderWindow &app, bool mouseLeftClick, sf::Vector2i mousePos) {
    update(app, mouseLeftClick, mousePos.x, mousePos.y);
}
