#include "Button.hpp"

TextButton::TextButton(float pos_x, float pos_y, int size,
                        std::string const &textname,
                        std::string const &fontname,
                        sf::Color const &col,
                        sf::Color const &litCol,
		                sf::Color const &clickedCol,
                        Align align,
                        sf::Text::Style style) :
    Button(),
    Message(pos_x,pos_y,size, textname, fontname, col, align, style)
{
    m_buttonColor = col;
    m_buttonLitColor = litCol;
    m_buttonClickedColor = clickedCol;
    m_buttonRect = m_messageText.getGlobalBounds();
}

TextButton::TextButton(float pos_x, float pos_y, int size,
                        std::wstring const &textname,
                        std::string const &fontname,
                        sf::Color const &col,
                        sf::Color const &litCol,
		                sf::Color const &clickedCol,
                        Align align,
                        sf::Text::Style style) :
    Button(),
    Message(pos_x,pos_y,size, textname, fontname, col, align, style)
{
    m_buttonColor = col;
    m_buttonLitColor = litCol;
    m_buttonClickedColor = clickedCol;
    m_buttonRect = m_messageText.getGlobalBounds();
}

TextButton::TextButton(sf::Vector2f pos, int size,
                    std::wstring const &text,
                    std::string const &fontname,
                    sf::Color const &col,
                    sf::Color const &litCol,
		            sf::Color const &clickedCol,
                    Align align,
                    sf::Text::Style style) :
        TextButton(pos.x, pos.y, size, text, fontname, col, litCol, clickedCol, align, style)
{}

void TextButton::lock() {
  m_isSelected = true;
  set_color(m_buttonClickedColor);
}

void TextButton::update(sf::RenderWindow &window, float const mouseX, float const mouseY){
    if (not is_selected()) {
        if ((m_buttonRect.contains(mouseX, mouseY))^is_lit()) {
            if (is_lit()) {
                set_color(m_buttonColor);
            }
            else {
                set_color(m_buttonLitColor);
            }
            m_isLit = !m_isLit;
        }
    }
    Message::update(window);
}

void TextButton::update(sf::RenderWindow &window, sf::Vector2f const mousePos){
    update(window, mousePos.x, mousePos.y);
}

void TextButton::update(sf::RenderWindow &window, sf::Vector2i const mousePos){
    update(window, mousePos.x, mousePos.y);
}

void TextButton::set_text(std::string const& key){
    Message::set_text(key);
    m_buttonRect = m_messageText.getGlobalBounds();
}

void TextButton::set_text(std::wstring const& text){
    Message::set_text(text);
    m_buttonRect = m_messageText.getGlobalBounds();
}
