#include <iostream>
#include "GraphicPlayer.hpp"
#include "SoundManager.hpp"
#include "Constants.hpp"

using namespace sf;

// __________________ Constructor _________________________

GraphicPlayer::GraphicPlayer() {}

GraphicPlayer::GraphicPlayer(sf::Vector2f positionPlayer, int orientationPlayer, double scalePlayer, int idPlayer) :
    m_handPlayer(positionPlayer,orientationPlayer, scalePlayer, idPlayer==0),
    m_tokenPlayer(positionPlayer + rotateVector(0,-DIST_HAND_BETBUBBLE, orientationPlayer) ,orientationPlayer,SCALE_TOKEN),
    m_betPlayer(positionPlayer + rotateVector(0, (-WIDTH_BETBUBBLE/2 - 50*(idPlayer%2)), orientationPlayer), idPlayer, orientationPlayer, 1.0)
{
    m_mainPlayer = (idPlayer == 0);

    m_isActive = false;
    m_mainPhase = false;

    //icon
    m_iconPlayer.setPosition(positionPlayer);
}

// _________________ Draw Function ________________________

void GraphicPlayer::draw(sf::RenderWindow &window){
    m_betPlayer.update(window);
    if (m_mainPhase) m_tokenPlayer.draw(window);
    m_handPlayer.draw(window);
    if (not m_mainPlayer) window.draw(m_iconPlayer);
}

// _________________ Event Function _______________________

std::shared_ptr<Card> GraphicPlayer::handle_mouse_click(float const mouseX, float const mouseY){
    if(m_isActive && m_mainPlayer){
        return m_handPlayer.play_card(mouseX, mouseY);
    }
    return nullptr;
}

std::shared_ptr<Card> GraphicPlayer::want_play(sf::Vector2f const mousePos){
    return m_handPlayer.want_play(mousePos);
}

std::shared_ptr<Card> GraphicPlayer::want_play(sf::Vector2i const mousePos){
    return m_handPlayer.want_play(mousePos);
}

std::shared_ptr<Card> GraphicPlayer::want_play(float const mouseX, float const mouseY){
    return m_handPlayer.want_play(mouseX, mouseY);
}

// _________________ Accesor Function ____________________

void GraphicPlayer::your_turn(){
    m_isActive = true;
    m_iconPlayer.your_turn();
}

void GraphicPlayer::end_turn(){
    m_isActive = false;
    m_iconPlayer.end_turn();
}

void GraphicPlayer::set_playable_cards(Core::Hand hand){
    m_handPlayer.set_playable_cards(hand);
}

void GraphicPlayer::sort_hand(Core::Trump trump){
    m_handPlayer.sort_hand(trump);
}

void GraphicPlayer::set_avatar(std::string const &spritename){
    m_iconPlayer.set_texture(spritename);
}


std::shared_ptr<Card> GraphicPlayer::play_card(Core::Card cardPlayed){
    soundManager.play_sound("card.wav");
    return m_handPlayer.play_card(cardPlayed);
}

void GraphicPlayer::set_play_zone(sf::Vector2f const position){
    m_handPlayer.set_play_zone(position);
}

void GraphicPlayer::give_card(){
    soundManager.play_sound("card_scatter.ogg");
    m_handPlayer.give_card();
}

void GraphicPlayer::give_card(Core::Card cardGiven){
    m_handPlayer.give_card(cardGiven);
}

void GraphicPlayer::give_trick(){
    //soundManager.play_sound("");
    m_tokenPlayer.new_trick();
}

void GraphicPlayer::draw_new_trick(){
    m_tokenPlayer.draw_new_trick();
}

sf::Vector2f GraphicPlayer::get_pos_free_token(){
    return m_tokenPlayer.get_pos_free_token();
}

void GraphicPlayer::begin_main_phase(){
    m_mainPhase = true;
    m_tokenPlayer.set_nb_token( m_betPlayer.get_bet().value );
    m_betPlayer.restart();
}

void GraphicPlayer::end_play_moment(){
    m_mainPhase = false;
}

void GraphicPlayer::set_bet(Core::Bet bet, bool draw){
    m_betPlayer.set_bet(bet, draw);
}

void GraphicPlayer::set_message(std::wstring text){
    m_betPlayer.set_message(text);
}

void GraphicPlayer::set_message(std::string textname){
    m_betPlayer.set_message(textname);
}

Core::Bet GraphicPlayer::get_bet(){
    return m_betPlayer.get_bet();
}

double GraphicPlayer::get_scale_token(){
    return m_tokenPlayer.get_scale();
}

bool GraphicPlayer::is_animated(){
    return (m_mainPhase && m_tokenPlayer.is_animated()) || m_betPlayer.is_animated();
}

void GraphicPlayer::restart_bet(){
    m_betPlayer.restart();
}

int GraphicPlayer::size_hand(){
    return m_handPlayer.size();
}
