#include "GraphicTimer.hpp"

sf::Color mix_color(sf::Color const A, sf::Color const B, int i){
    sf::Color res(0,0,0,0);
    res.r = (A.r * (255-i) + B.r * i)/255;
    res.g = (A.g * (255-i) + B.g * i)/255;
    res.b = (A.b * (255-i) + B.b * i)/255;
    res.a = (A.a * (255-i) + B.a * i)/255;
    return res;
}

GraphicTimer::GraphicTimer(sf::Vector2f center,
                           int size, int sizeOutline,
                           sf::Color begin, sf::Color end) :
m_size(size), m_nbPointsDrawn(2), m_nbPoints(255), m_center(center)
{
    for(unsigned int i=0; i<m_nbPoints; i++) {
        m_vertices.emplace_back(getPoint(i, m_size, sizeOutline), mix_color(begin, end, i));
    }
}

GraphicTimer::GraphicTimer(int x, int y, int size, int sizeOutline, sf::Color begin, sf::Color end) :
    GraphicTimer(sf::Vector2f(x,y), size, sizeOutline, begin, end) {}

sf::Vector2f GraphicTimer::getPoint(std::size_t index, int size, int sizeOutline) const {
    if(index%2) {
        return getPoint(index-1, size-sizeOutline, 0);
    }
    index/=2;
    double angle = 4*M_PI / (m_nbPoints-1);
    double y = -cos(angle*index)*size;
    double x =  sin(angle*index)*size;
    return sf::Vector2f(x,y)+m_center;
}

void GraphicTimer::update(sf::RenderWindow& app){
    app.draw(&m_vertices[0],m_nbPointsDrawn,sf::TrianglesStrip);
}

void GraphicTimer::reduce(){
    m_nbPointsDrawn+=2;
    if(m_nbPointsDrawn > m_nbPoints) {
        m_nbPointsDrawn = 2;
    }
}
