#include "PlayerDisplay.hpp"

using namespace sf;

PlayerDisplay::PlayerDisplay(float x, float y, int iconSize, int fontSize,
			     std::wstring const &name,
			     std::string const &fontname,
			     sf::Color const &col,
			     sf::Text::Style style) :
	m_icon(iconSize),
    m_name(x, y, fontSize, name, fontname, col, Align::LEFT, style),
	m_offset(iconSize/2+10, -iconSize/4)
{
	sf::Vector2f pos(x,y);
	m_icon.set_position(pos);
	m_name.set_position(pos+m_offset);
}

PlayerDisplay::PlayerDisplay(std::string const &spriteName,
			     float x, float y, int iconSize, int fontSize,
			     std::wstring const &name,
			     std::string const &fontname,
			     sf::Color const &col,
			     sf::Text::Style style) :
m_icon(spriteName, iconSize),
m_name(x, y, fontSize, name, fontname, col, Align::LEFT, style),
m_offset(iconSize/2, -iconSize/4)
{
	sf::Vector2f pos(x,y);
	m_icon.set_position(pos);
	m_name.set_position(pos+m_offset);
}


void PlayerDisplay::update(sf::RenderWindow &app, float const mouseX, float const mouseY) {
	m_icon.update(app);
	m_name.update(app);
}

void PlayerDisplay::update(sf::RenderWindow &app, sf::Vector2f const mousePos){
	update(app, mousePos.x, mousePos.y);
}

void PlayerDisplay::update(sf::RenderWindow &app, sf::Vector2i const mousePos) {
	update(app, mousePos.x, mousePos.y);
}

void PlayerDisplay::your_turn(){
    m_icon.your_turn();
}

void PlayerDisplay::end_turn(){
    m_icon.end_turn();
}

void PlayerDisplay::set_texture(std::string const &spritename){
  m_icon.set_texture(spritename);
}

void PlayerDisplay::set_text(std::wstring const &newText) {
  m_name.set_text(newText);
}

void PlayerDisplay::set_text(std::string const &key) {
  m_name.set_text(key);
}

void PlayerDisplay::set_font_size(int size) {
	m_name.set_font_size(size);
}

int PlayerDisplay::get_name_width(){
	return m_name.get_width();
}

void PlayerDisplay::set_position(float const imageX, float const imageY, int angle){
	set_position(sf::Vector2f(imageX,imageY), angle);
}

void PlayerDisplay::set_position(sf::Vector2f const positionImage, int const angle){
  m_icon.set_position(positionImage, angle);
  m_name.set_position(positionImage+m_offset, angle);
}

void PlayerDisplay::move(float const x, float const y){
	move(sf::Vector2f(x,y));
}

void PlayerDisplay::move(sf::Vector2f const offset){
	sf::Vector2f currentPos(m_icon.getPosition());
	set_position(currentPos+offset);
}
