#include "ScoreDisplay.hpp"

#include <iostream> // DEBUG

int const EndGameDisplay::WIDTH_CELL=30;
sf::Vector2f const EndGameDisplay::WHOWINS_OFFSET(260,25);

EndGameDisplay::EndGameDisplay() :
    ScoreDisplay::ScoreDisplay(),
    m_whoWins(WHOWINS_OFFSET.x, WHOWINS_OFFSET.y, WIDTH_AVATAR, FONT_SIZE, L"", "MPlantin_0.ttf", WHITE)
{
    m_okButton.set_text("backToMenu");
    /** /!\ Position of every object is given in relation to the ScoreDisplay position.
            It is shifted at the end to fit in the background **/

    for (int i = 0 ; i< 4 ; i++) { // 4 horizontal lines
        m_lines.emplace_back(sf::Vector2f(ScoreDisplay::WIDTH-20,1));
        m_lines[i].setPosition(sf::Vector2f(10,ScoreDisplay::TOP_ZONE_HEIGHT+(i+1)*ScoreDisplay::CELL_HEIGHT));
    }

    // 11 vertical lines
    for (int i = 0 ; i<11 ; i++) {
        m_lines.emplace_back(sf::Vector2f(1, 5*ScoreDisplay::CELL_HEIGHT-20));
        m_lines[i+4].setPosition(sf::Vector2f(130+i*WIDTH_CELL, ScoreDisplay::TOP_ZONE_HEIGHT+20));
    }

    // WHO wins
    m_infoMessages.emplace_back(WHOWINS_OFFSET-sf::Vector2f(20,10), ScoreDisplay::FONT_SIZE, L"Winner : ","MPlantin_0.ttf", WHITE,  Align::RIGHT);
    // headers of table
    for (unsigned int i = 0 ; i< Core::Kernel::nbCardsPerTurn.size() ; i++){
        m_infoMessages.emplace_back(sf::Vector2f(130+i*WIDTH_CELL+WIDTH_CELL/2,TOP_ZONE_HEIGHT+CELL_HEIGHT/2), ScoreDisplay::FONT_SIZE, std::to_wstring(Core::Kernel::nbCardsPerTurn[i]), "MPlantin_0.ttf", WHITE, Align::CENTER);
    }
    m_infoMessages.emplace_back(sf::Vector2f(480,TOP_ZONE_HEIGHT+CELL_HEIGHT/2), ScoreDisplay::FONT_SIZE, "total","MPlantin_0.ttf", WHITE,  Align::CENTER);

    // Players avatars. NO NAMES BECAUSE NOT ENOUGH ROOM
    sf::Vector2f posPlayerSouth(60,TOP_ZONE_HEIGHT+3*CELL_HEIGHT/2);
    m_playerDisplays[0].set_text(L"");
    m_playerDisplays[0].set_position(posPlayerSouth);

    sf::Vector2f posPlayerWest(60,TOP_ZONE_HEIGHT+5*CELL_HEIGHT/2);
    m_playerDisplays[1].set_text(L"");
    m_playerDisplays[1].set_position(posPlayerWest);

    sf::Vector2f posPlayerNorth(60,TOP_ZONE_HEIGHT+7*CELL_HEIGHT/2);
    m_playerDisplays[2].set_text(L"");
    m_playerDisplays[2].set_position(posPlayerNorth);

    sf::Vector2f posPlayerEast(60,TOP_ZONE_HEIGHT+9*CELL_HEIGHT/2);
    m_playerDisplays[3].set_text(L"");
    m_playerDisplays[3].set_position(posPlayerEast);

    // All score messages
    for (unsigned int rnd = 0 ; rnd < Core::Kernel::nbCardsPerTurn.size() ; rnd++){
        for (int i = 0 ; i<4 ; i++){
            m_scoreMessages.emplace_back(130+rnd*WIDTH_CELL+WIDTH_CELL/3, ScoreDisplay::TOP_ZONE_HEIGHT+(i+1)*CELL_HEIGHT+CELL_HEIGHT/3, ScoreDisplay::FONT_SIZE, L"0", "MPlantin_0.ttf",WHITE, Align::LEFT);
        }
    }
    for (int i = 0 ; i< 4 ; i++){
        m_totalMessages.emplace_back(460, ScoreDisplay::TOP_ZONE_HEIGHT+(i+1)*CELL_HEIGHT+CELL_HEIGHT/3, ScoreDisplay::FIGURE_FONT_SIZE, L"0", "MPlantin_0.ttf", WHITE, Align::LEFT);
    }

    // _______ Apply change of origin to every object built so far ________
    // Everything shifted to fit in the ScoreDisplay rectangle
    for (auto& line : m_lines){
        line.move(ScoreDisplay::POSITION);
        line.setFillColor(WHITE);
    }
    for (auto& message : m_infoMessages){
        message.move(ScoreDisplay::POSITION);
    }
    for (auto& message : m_scoreMessages){
        message.move(ScoreDisplay::POSITION);
    }
    for (auto& message : m_totalMessages) {
        message.move(ScoreDisplay::POSITION);
    }
    for (auto& display : m_playerDisplays){
        display.move(ScoreDisplay::POSITION);
    }
    m_whoWins.move(ScoreDisplay::POSITION);
}

void EndGameDisplay::update(sf::RenderWindow &window, sf::Vector2f const mousePos){
    update(window, mousePos.x, mousePos.y);
}

void EndGameDisplay::update(sf::RenderWindow &window, sf::Vector2i const mousePos){
    update(window, mousePos.x, mousePos.y);
}

void EndGameDisplay::update(sf::RenderWindow &window, float const mouseX, float const mouseY){
    ScoreDisplay::draw(window);
    for (auto& message : m_totalMessages) {
        message.update(window);
    }
    for (auto& message : m_scoreMessages) {
        message.update(window);
    }
    for (auto& sprite : m_playerDisplays) {
        sprite.update(window);
    }
    m_okButton.update(window, mouseX, mouseY);
    m_whoWins.update(window);
}

void EndGameDisplay::retrieve_bet_info(Core::Trump const trump, std::array<int,4> const& bets, bool const isMechoune, bool const isChoune){
    ScoreDisplay::retrieve_bet_info(trump, bets, isMechoune, isChoune);
    if (isChoune) {
        for (int p = 0 ; p<4 ; p++) {
            m_scoreMessages[4*(m_nRound-1) + p].set_color(RED);
        }
    } else if (isMechoune && !isChoune) {
        for (int p = 0 ; p<4 ; p++) {
            m_scoreMessages[4*(m_nRound-1)+ p].set_color(YELLOW);
        }
    }
}

void EndGameDisplay::retrieve_score_info(std::array<int,4> const& roundResults){
    ScoreDisplay::retrieve_score_info(roundResults);
    for (int p = 0 ; p < 4 ; p++){
        m_totalMessages[p].set_text(std::to_wstring(m_totalScore[p]));
    }
    for (int p = 0 ; p < 4 ; p++) {
        m_scoreMessages[4*(m_nRound-2)+p].set_text(std::to_wstring(m_roundScore[p]));
    }
}

void EndGameDisplay::retrieve_winner(int const winner) {
    switch (winner) {
        case 0:
            m_whoWins.set_texture("faces/chat.png");
            if (optionManager()->is_online()){
                m_whoWins.set_text(optionManager()->get_player_name().toWideString());
            } else {
                m_whoWins.set_text("you");
            }
            break;
        case 1:
            m_whoWins.set_texture("faces/roi-trefle-0.png");
            m_whoWins.set_text("west");
            break;
        case 2:
            m_whoWins.set_texture("faces/jongleur-coeur-0.png");
            m_whoWins.set_text("north");
            break;
        case 3:
        default:
            m_whoWins.set_texture("faces/cavalier-carreau-0.png");
            m_whoWins.set_text("east");
            break;
    }
}
