#include "ScoreDisplay.hpp"

sf::Vector2f const EndRoundDisplay::TRUMP_OFFSET(380,25);
sf::Vector2f const EndRoundDisplay::ROUND_OFFSET(200,25);
sf::Vector2f const EndRoundDisplay::NBCARD_OFFSET(20,25);
int const EndRoundDisplay::WIDTH_TRUMP=40;


EndRoundDisplay::EndRoundDisplay() :
    ScoreDisplay::ScoreDisplay(),
    m_trumpSprite(WIDTH_TRUMP,WIDTH_TRUMP,false)
{
    m_okButton.set_text("nextRound");

    /** /!\ Position of every object is given in relation to the ScoreDisplay position.
            It is shifted at the end to fit in the background **/

    for (int i = 0 ; i< 4 ; i++) { // 4 horizontal lines
        m_lines.emplace_back(sf::Vector2f(ScoreDisplay::WIDTH-20,1));
        m_lines[i].setPosition(sf::Vector2f(10,ScoreDisplay::TOP_ZONE_HEIGHT+(i+1)*ScoreDisplay::CELL_HEIGHT));
    }

    // 3 vertical lines
    m_lines.emplace_back(sf::Vector2f(1, 5*ScoreDisplay::CELL_HEIGHT));
    m_lines[4].setPosition(sf::Vector2f(200, ScoreDisplay::TOP_ZONE_HEIGHT));

    m_lines.emplace_back(sf::Vector2f(1, 5*ScoreDisplay::CELL_HEIGHT));
    m_lines[5].setPosition(sf::Vector2f(310, ScoreDisplay::TOP_ZONE_HEIGHT));

    m_lines.emplace_back(sf::Vector2f(1, 5*ScoreDisplay::CELL_HEIGHT));
    m_lines[6].setPosition(sf::Vector2f(420, ScoreDisplay::TOP_ZONE_HEIGHT));

    // trump display (position 0 in vector)
    m_infoMessages.emplace_back(TRUMP_OFFSET, ScoreDisplay::FONT_SIZE, "trump","MPlantin_0.ttf", WHITE, Align::LEFT);
    m_trumpSprite.set_position(TRUMP_OFFSET+ sf::Vector2f(m_infoMessages[0].get_width()+10,-15) + ScoreDisplay::POSITION);

    // number of round display (position 1 in vector)
    m_infoMessages.emplace_back(ROUND_OFFSET, ScoreDisplay::FONT_SIZE, "round", "MPlantin_0.ttf", WHITE, Align::LEFT);
    sf::Vector2f nRoundMessagePos(ROUND_OFFSET+sf::Vector2f(m_infoMessages[1].get_width()+10,0));
    m_nRoundMessage = Message(nRoundMessagePos, ScoreDisplay::FONT_SIZE, std::to_wstring(m_nRound)+L"/10", "MPlantin_0.ttf", WHITE, Align::LEFT),

    // number of cards display (position 2 in vector)
    m_infoMessages.emplace_back(NBCARD_OFFSET, ScoreDisplay::FONT_SIZE, "cards","MPlantin_0.ttf", WHITE, Align::LEFT);
    sf::Vector2f nCardMessagePos(NBCARD_OFFSET+sf::Vector2f(m_infoMessages[2].get_width()+10,0));
    m_nCardMessage = Message(nCardMessagePos, ScoreDisplay::FONT_SIZE, std::to_wstring(Core::Kernel::nbCardsPerTurn[m_nRound-1]), "MPlantin_0.ttf", WHITE, Align::LEFT),

    // mechoune / choune display
    m_meChouneMessage = Message(20,TOP_ZONE_HEIGHT+CELL_HEIGHT/2-5, ScoreDisplay::FONT_SIZE, L"", "MPlantin_0.ttf", WHITE, Align::LEFT);

    // headers of table
    m_infoMessages.emplace_back(sf::Vector2f(250,TOP_ZONE_HEIGHT+CELL_HEIGHT/2-10), ScoreDisplay::FONT_SIZE, "bet","MPlantin_0.ttf", WHITE,  Align::CENTER);
    m_infoMessages.emplace_back(sf::Vector2f(360,TOP_ZONE_HEIGHT+CELL_HEIGHT/2-10), ScoreDisplay::FONT_SIZE, "score","MPlantin_0.ttf", WHITE,  Align::CENTER);
    m_infoMessages.emplace_back(sf::Vector2f(470,TOP_ZONE_HEIGHT+CELL_HEIGHT/2-10), ScoreDisplay::FONT_SIZE, "total","MPlantin_0.ttf", WHITE,  Align::CENTER);

    // Players display
    sf::Vector2f posPlayerSouth(40,TOP_ZONE_HEIGHT+3*CELL_HEIGHT/2);
    if (optionManager()->is_online()){
        m_playerDisplays[0].set_text(optionManager()->get_player_name().toWideString());
        //m_playerDisplays[0].set_font_size(m_playerDisplays[0].get_name_width());
    } else {
        m_playerDisplays[0].set_text("you");
    }
    m_playerDisplays[0].set_position(posPlayerSouth);

    sf::Vector2f posPlayerWest(40,TOP_ZONE_HEIGHT+5*CELL_HEIGHT/2);
    m_playerDisplays[1].set_text("west");
    m_playerDisplays[1].set_position(posPlayerWest);

    sf::Vector2f posPlayerNorth(40,TOP_ZONE_HEIGHT+7*CELL_HEIGHT/2);
    m_playerDisplays[2].set_text("north");
    m_playerDisplays[2].set_position(posPlayerNorth);

    sf::Vector2f posPlayerEast(40,TOP_ZONE_HEIGHT+9*CELL_HEIGHT/2);
    m_playerDisplays[3].set_text("east");
    m_playerDisplays[3].set_position(posPlayerEast);

    for (int i = 0 ; i< 4 ; i++) {
        m_betMessages.emplace_back(250, ScoreDisplay::TOP_ZONE_HEIGHT+(i+1)*CELL_HEIGHT+CELL_HEIGHT/3, ScoreDisplay::FIGURE_FONT_SIZE, L"0", "MPlantin_0.ttf", WHITE, Align::LEFT);
        m_scoreMessages.emplace_back(360, ScoreDisplay::TOP_ZONE_HEIGHT+(i+1)*CELL_HEIGHT+CELL_HEIGHT/3, ScoreDisplay::FIGURE_FONT_SIZE, L"0", "MPlantin_0.ttf", WHITE, Align::LEFT);
        m_totalMessages.emplace_back(470, ScoreDisplay::TOP_ZONE_HEIGHT+(i+1)*CELL_HEIGHT+CELL_HEIGHT/3, ScoreDisplay::FIGURE_FONT_SIZE, L"0", "MPlantin_0.ttf", WHITE, Align::LEFT);
    }

    // _______ Apply change of origin to every object built so far ________
    for (auto& line : m_lines){
        line.move(ScoreDisplay::POSITION); // Everything shifted to fit in the ScoreDisplay rectangle
        line.setFillColor(WHITE);
    }
    for (auto& message : m_infoMessages){
        message.move(ScoreDisplay::POSITION);
    }
    for (auto& message : m_scoreMessages){
        message.move(ScoreDisplay::POSITION);
    }
    for (auto& message : m_betMessages){
        message.move(ScoreDisplay::POSITION);
    }
    for (auto& message : m_totalMessages){
        message.move(ScoreDisplay::POSITION);
    }
    for (auto& display : m_playerDisplays){
        display.move(ScoreDisplay::POSITION);
    }
    m_nCardMessage.move(ScoreDisplay::POSITION);
    m_nRoundMessage.move(ScoreDisplay::POSITION);
    m_meChouneMessage.move(ScoreDisplay::POSITION);
}

void EndRoundDisplay::update(sf::RenderWindow &window, sf::Vector2f const mousePos){
    update(window, mousePos.x, mousePos.y);
}

void EndRoundDisplay::update(sf::RenderWindow &window, sf::Vector2i const mousePos){
    update(window, mousePos.x, mousePos.y);
}

void EndRoundDisplay::update(sf::RenderWindow &window, float const mouseX, float const mouseY){
    ScoreDisplay::draw(window);
    for(auto& message : m_scoreMessages){
        message.update(window);
    }
    for(auto& message : m_betMessages){
        message.update(window);
    }
    for(auto& message : m_totalMessages){
        message.update(window);
    }
    for (auto& sprite : m_playerDisplays) {
        sprite.update(window);
    }
    m_nRoundMessage.update(window);
    m_nCardMessage.update(window);
    m_meChouneMessage.update(window);
    m_okButton.update(window, mouseX, mouseY);
    m_trumpSprite.update(window);
}

void EndRoundDisplay::retrieve_bet_info(Core::Trump const trump, std::array<int,4> const& bets, bool const isMechoune, bool const isChoune){
    ScoreDisplay::retrieve_bet_info(trump, bets, isMechoune, isChoune);
    if (isChoune) {
        m_meChouneMessage.set_text(L"Choune");
        m_meChouneMessage.set_color(RED);
    } else if (isMechoune && !isChoune) {
        m_meChouneMessage.set_text(L"Mechoune");
        m_meChouneMessage.set_color(YELLOW);
    } else {
        m_meChouneMessage.set_text(L"");
    }
    if (m_nRound==10){
        m_okButton.set_text("next");
    }
    m_trumpSprite.set_texture("sprites_token/" + Core::trump_to_string(trump) + ".png");
    for(int i=0 ; i<4 ; i++){
        m_betMessages[i].set_text(std::to_wstring(m_roundBet[i]));
    }
}

void EndRoundDisplay::retrieve_score_info(std::array<int,4> const& roundResults){
    m_nRoundMessage.set_text(std::to_wstring(m_nRound)+L"/10");
    m_nCardMessage.set_text(std::to_wstring(Core::Kernel::nbCardsPerTurn[m_nRound-1]));
    ScoreDisplay::retrieve_score_info(roundResults);
    for(int p=0 ; p<4 ; p++){
        m_scoreMessages[p].set_text(std::to_wstring(m_roundScore[p]));
        m_totalMessages[p].set_text(std::to_wstring(m_totalScore[p]));
    }
}
