#include "ScoreDisplay.hpp"
#include "OptionManager.hpp"

#include <string>

int const ScoreDisplay::WIDTH=540;
int const ScoreDisplay::HEIGHT=380;

int const ScoreDisplay::CELL_HEIGHT=50;
int const ScoreDisplay::TOP_ZONE_HEIGHT=60;
int const ScoreDisplay::WIDTH_AVATAR=40;
int const ScoreDisplay::FONT_SIZE=25;
int const ScoreDisplay::FIGURE_FONT_SIZE=40;

sf::Vector2f const ScoreDisplay::POSITION(optionManager()->get_screen_width()/2 - WIDTH/2, optionManager()->get_screen_height()/2 - HEIGHT/2); // top left corner
int const ScoreDisplay::OK_BUTTON_OFFSET=40;
sf::Color const ScoreDisplay::BG_COLOR(0,0,0,192);

using namespace std;

ScoreDisplay::ScoreDisplay() :
    m_nRound(1),
    m_okButton(WIDTH/2+POSITION.x, HEIGHT+POSITION.y-OK_BUTTON_OFFSET, 40, L"","MPlantin_0.ttf",GRAY,WHITE,WHITE),
    m_background(sf::Vector2f(WIDTH,HEIGHT)),
    m_totalScore{0,0,0,0},
    m_roundScore{-1,-1,-1,-1},
    m_roundBet{-1,-1,-1,-1}
{
    m_background.setFillColor(BG_COLOR);
    m_background.setPosition(POSITION);
    for (int i = 0 ; i<4 ; i++) {
        m_playerDisplays.emplace_back(0,0,WIDTH_AVATAR, FONT_SIZE-2, L"", "MPlantin_0.ttf", WHITE);
    }
    m_playerDisplays[0].set_texture("faces/chat.png");
    m_playerDisplays[1].set_texture("faces/roi-trefle-0.png");
    m_playerDisplays[2].set_texture("faces/jongleur-coeur-0.png");
    m_playerDisplays[3].set_texture("faces/cavalier-carreau-0.png");
}

void ScoreDisplay::draw(sf::RenderWindow& window){
    window.draw(m_background);
    for (auto& message : m_infoMessages){
        message.update(window);
    }
    for (auto& line : m_lines){
        window.draw(line);
    }
}

std::array<int,4> ScoreDisplay::correct_array_offset(int const n, std::array<int,4> const& src){
    std::array<int,4> result = {0,0,0,0};
    for (int i = 0 ; i<4 ; i++) {
        result[i] = src[(i+n)%4];
    }
    return result;
}

void ScoreDisplay::retrieve_bet_info(Core::Trump const trump, array<int,4> const& bets, bool const isMechoune, bool const isChoune){
    m_roundBet = bets; // nb_tricks_objective is not messed up
    m_historyTrumps.push_back(trump);
    m_historyMechoune.push_back(isMechoune);
    m_historyChoune.push_back(isChoune);
}

void ScoreDisplay::retrieve_score_info(array<int,4> const& scores){
    for (unsigned int i = 0 ; i < m_roundScore.size() ; i++) {
        m_roundScore[i] = scores[i] - m_totalScore[i];
    }
    m_totalScore = scores;
    m_historyScore.push_back(m_roundScore);
    m_nRound++;
}

bool ScoreDisplay::okButton_lit(){
    return m_okButton.is_lit();
}
