
#include "TokenArray.hpp"

using namespace sf;

// ____________________ Constructor ____________________

TokenArray::TokenArray(){}

TokenArray::TokenArray(sf::Vector2f position, int orientation, double scale){
    m_centerArray = position;
	m_orientationArray = orientation;
	m_scaleArray = scale;

    m_nbToken = 0;
    m_nbTrick = 0;
    m_nbTrickDraw = 0;
}

void TokenArray::set_nb_token(int nb){
    m_nbToken = nb;
    m_nbTrick = 0;
    m_nbTrickDraw = 0;

    m_tokenPlayer = std::vector<Token> (std::max(1,nb));
    
    for(unsigned int i=0; i<m_tokenPlayer.size(); i++){
        m_tokenPlayer[i].set_scale(m_scaleArray);
        m_tokenPlayer[i].set_position(WINDOW_WIDTH/2, WINDOW_HEIGHT/2);
    }
    
    m_cardToken = std::vector<Card> (0);
    if(nb == 0){
        m_tokenPlayer[0].set_black();
    }

    replace_token(30);
}

// ___________________ Draw Function ____________________

void TokenArray::draw(sf::RenderWindow &window){
    if(m_nbToken == 0){ //particular case
        m_tokenPlayer[0].continue_move();
        window.draw(m_tokenPlayer[0]);
        for(int i=0; i<m_nbTrickDraw; i++){
            m_cardToken[i].continue_move();
            window.draw(m_cardToken[i]);
        }
    }
    else{
        for(int i=0; i<m_nbTrickDraw; i++){
            m_cardToken[i].continue_move();
            window.draw(m_cardToken[i]);
        }
        for(int i=0; i<m_nbToken; i++){
            m_tokenPlayer[i].continue_move();
            window.draw(m_tokenPlayer[i]);
        }
    }
}

// ____________________ Move Function ____________________

sf::Vector2f TokenArray::compute_position(int i, int n){
    return m_centerArray + rotateVector(m_scaleArray*(0.5 + double(i) - double(n)/2.0)*WIDTH_CARD,0, m_orientationArray);
}

void TokenArray::replace_token(int time){ //replace token
    if(m_nbToken == 0){ //particular case
        int n = 1 + m_nbTrick;
        
        if(n>4){
            m_scaleArray = SCALE_TOKEN * 4.0 / n;
        }
        else{
            m_scaleArray = SCALE_TOKEN;
        }
        
        m_tokenPlayer[0].zoom_sprite(m_scaleArray,time);
        m_tokenPlayer[0].move_sprite_in_time(compute_position(0,n),360-m_orientationArray, time);
        for(int i=0; i<m_nbTrickDraw; i++){
            m_cardToken[i].move_sprite_in_time(compute_position(i+1,n),360-m_orientationArray, time);
            m_cardToken[i].zoom_sprite(m_scaleArray,time);
        }
    }
    else{
        int n = std::max(m_nbToken , m_nbTrick);
        
        if(n>4){
            m_scaleArray = SCALE_TOKEN * 4.0 / n;
        }
        else{
            m_scaleArray = SCALE_TOKEN;
        }
        
        for(int i=0; i<m_nbTrickDraw; i++){
            m_cardToken[i].move_sprite_in_time(compute_position(i,n),360-m_orientationArray, time);
            m_cardToken[i].zoom_sprite(m_scaleArray,time);
        }
        for(int i=0; i<m_nbToken; i++){
            m_tokenPlayer[i].move_sprite_in_time(compute_position(i,n),360-m_orientationArray, time);
            m_tokenPlayer[i].zoom_sprite(m_scaleArray,time);
        }
    }
}

// _____________________ Trick Function _________________

void TokenArray::new_trick(){
    m_nbTrick ++;
    m_cardToken.push_back(Card());
    replace_token(C_TRICK_TIME);
}

void TokenArray::draw_new_trick(){
    int n = std::max(m_nbToken , m_nbTrick);
    int i = m_nbTrickDraw;
    if(!m_nbToken){
        n++;
        i++;
    }
    m_cardToken[m_nbTrickDraw].set_position(compute_position(i,n),360-m_orientationArray);
    m_cardToken[m_nbTrickDraw].set_scale(m_scaleArray);
    m_nbTrickDraw ++;
}

sf::Vector2f TokenArray::get_pos_free_token(){
    int n = std::max(m_nbToken , m_nbTrick);
    int i = m_nbTrick-1;
    if(!m_nbToken){
        n++;
        i++;
    }
    return compute_position(i,n);
}

double TokenArray::get_scale(){
    return m_scaleArray;
}

bool TokenArray::is_animated(){
    for(int i=0; i<m_nbToken; i++){
        if(m_tokenPlayer[i].is_animated()){
            return true;
        }
    }
    return false;
}
