#include "GUI.hpp"
#include "enumType.hpp"
#include "TextManager.hpp"
#include "OptionManager.hpp"
#include "AnimatedSprite.hpp"
#include <vector>
#include <string>

void GUI::run_credits_loop() {

		int const CONTRIBUTOR_FONT_SIZE = 22*SCALE_RESOLUTION_HEIGHT;
		int const CREDIT_FONT_SIZE = 25*SCALE_RESOLUTION_HEIGHT;


		textManager.load("/credits/"+optionManager()->get_language_string()+".txt");

		Background theBackground("menu_background.png");
		TextButton backButton(WINDOW_WIDTH/2, WINDOW_HEIGHT-(80*WINDOW_WIDTH)/1080, MENU_BUTTON_FONT_SIZE, "back", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);

		std::vector<Message> allMessages;
		AnimatedSprite cartomLogo(130*SCALE_RESOLUTION,130*SCALE_RESOLUTION);
		cartomLogo.set_texture("logoCartomensia.png");
		cartomLogo.set_position(100*SCALE_RESOLUTION, 100*SCALE_RESOLUTION);
		allMessages.emplace_back(190*SCALE_RESOLUTION, 65*SCALE_RESOLUTION, CONTRIBUTOR_FONT_SIZE, "creditCartom1","MPlantin_0.ttf", WHITE, Align::LEFT);
		allMessages.emplace_back(190*SCALE_RESOLUTION, 65*SCALE_RESOLUTION + CONTRIBUTOR_FONT_SIZE+ 10*SCALE_RESOLUTION_HEIGHT, CONTRIBUTOR_FONT_SIZE, "creditCartom2","MPlantin_0.ttf", WHITE, Align::LEFT);
		allMessages.emplace_back(190*SCALE_RESOLUTION, 65*SCALE_RESOLUTION + 2*(CONTRIBUTOR_FONT_SIZE+ 10*SCALE_RESOLUTION_HEIGHT), CONTRIBUTOR_FONT_SIZE, "creditCartom3","MPlantin_0.ttf", WHITE, Align::LEFT);
		TextButton cartomWS(190*SCALE_RESOLUTION, 65*SCALE_RESOLUTION + 3*(CONTRIBUTOR_FONT_SIZE+ 10*SCALE_RESOLUTION_HEIGHT), CONTRIBUTOR_FONT_SIZE, "creditCartomWeb", "MPlantin_0.ttf", WHITE, GRAY, BLACK, Align::LEFT);

		AnimatedSprite batardeLogo(100*SCALE_RESOLUTION,150*SCALE_RESOLUTION);
		batardeLogo.set_texture("logoBatarde.png");
		batardeLogo.set_position(WINDOW_WIDTH-130*SCALE_RESOLUTION, 60+190*SCALE_RESOLUTION_HEIGHT);
		allMessages.emplace_back(WINDOW_WIDTH-220*SCALE_RESOLUTION, 110+150*SCALE_RESOLUTION_HEIGHT, CONTRIBUTOR_FONT_SIZE, "creditBatarde","MPlantin_0.ttf", WHITE, Align::RIGHT);
		TextButton batardeWS(WINDOW_WIDTH-220*SCALE_RESOLUTION, 110+150*SCALE_RESOLUTION_HEIGHT + CONTRIBUTOR_FONT_SIZE+ 10*SCALE_RESOLUTION_HEIGHT, CONTRIBUTOR_FONT_SIZE, "creditBatardeWeb","MPlantin_0.ttf", WHITE, GRAY, BLACK, Align::RIGHT);

		//Contributors
		allMessages.emplace_back(WINDOW_WIDTH/2, WINDOW_HEIGHT/2+20*SCALE_RESOLUTION_HEIGHT, CREDIT_FONT_SIZE,"creditTeamMember", "MPlantin_0.ttf",WHITE, Align::CENTER, sf::Text::Bold);
		for (int i = 0 ; i<18 ; i++){ // 18 members of the project :)
			allMessages.emplace_back(WINDOW_WIDTH/2+(i/6-1)*250*SCALE_RESOLUTION_HEIGHT,
									 WINDOW_HEIGHT/2+90+(i%6)*30*SCALE_RESOLUTION_HEIGHT,
									 CONTRIBUTOR_FONT_SIZE,
								     "creditName"+std::to_string(i),
									 "MPlantin_0.ttf", WHITE, Align::CENTER);
		}

		// Special Thanks
		allMessages.emplace_back(WINDOW_WIDTH/2, WINDOW_HEIGHT/2-10*SCALE_RESOLUTION_HEIGHT, CONTRIBUTOR_FONT_SIZE, "creditSpecialThanks", "MPlantin_0.ttf", WHITE);

		sf::Event event;
		sf::Vector2i mousePos;

		while (app.isOpen()) {

			 mousePos = sf::Mouse::getPosition(app);

			 // #1 Event handling
			 while(app.pollEvent(event)) {
				 // check the type of the event...

				 switch (event.type) {
					// window closed
					case sf::Event::Closed:
						app.close();

					// key pressed
					case sf::Event::KeyPressed: {
						if(event.key.code == sf::Keyboard::Escape)
							return;
						break;
					}

				   case sf::Event::MouseButtonPressed :
					   if (event.mouseButton.button == sf::Mouse::Left) {
							if (backButton.is_lit()) {
								soundManager.play_sound("click.wav");
								return;
							} else if (cartomWS.is_lit()) {
								soundManager.play_sound("click.wav");
								#ifdef _WIN32
									ShellExecute(0, 0, L"http://graal.ens-lyon.fr/cartomensia/", 0, 0 , SW_SHOW );
								#elif __unix__
									system("sensible-browser http://graal.ens-lyon.fr/cartomensia/");
								#endif
							} else if (batardeWS.is_lit()) {
								soundManager.play_sound("click.wav");
								#ifdef _WIN32
									ShellExecute(0, 0, L"http://www.labatarde.com/", 0, 0 , SW_SHOW );
								#elif __unix__
									system("sensible-browser http://www.labatarde.com/");
								#endif
							}

					   }
					   break;

					// we don't process other types of events
					default:
						break;
				 }
			 }

			 // #2 Various objects update
			 theBackground.update(app);
			 backButton.update(app,mousePos);
			 cartomWS.update(app,mousePos);
			 batardeWS.update(app,mousePos);

			 cartomLogo.update(app);
			 batardeLogo.update(app);

			 for (auto & mess : allMessages) {
				 mess.update(app);
			 }
			 m_trophiesDrawer.update(app);

			 // #3 Screen display
			 app.display();
		 }

}
