#include <thread>
#include <iostream>

#include "GUI.hpp"
#include "IA.hpp"
#include "Core.hpp"

#include "BetArray.hpp"
#include "DropDownMenu.hpp"
#include "GraphicGame.hpp"
#include "GraphicTimer.hpp"

using namespace std;

void GUI::run_game_loop() {

    srand(time(NULL));
    soundManager.play_music("main_music.wav");

    int nbDesc;
    switch (optionManager()->get_difficulty()) {
        case Difficulty::EASY:
            nbDesc=50;
            break;
        case Difficulty::HARD:
            nbDesc=500;
            break;
        case Difficulty::MEDIUM:
        default:
            nbDesc=200;
            break;
    }

    Core::ParallelPlayer * gui_player = new Core::ParallelPlayer();
    IA::IAPlayer * ia_player1 = new IA::IAPlayer(nbDesc);
    IA::IAPlayer * ia_player2 = new IA::IAPlayer(nbDesc);
    IA::IAPlayer * ia_player3 = new IA::IAPlayer(nbDesc);
    auto kernel = make_shared<Core::Kernel>(gui_player, ia_player1, ia_player2, ia_player3);
    thread thread_kernel(&Core::run_kernel, kernel);

    GraphicGame gameManager(&kernel->get_game_state(0), gui_player);

	Background theBackground("tapis.png");

	DropDownMenu menu(&kernel->get_game_state(0));

	// Event handling
	sf::Event event;

	// Cursor
	sf::Vector2i mousePos;
	bool clicked  = false;

	statManager.begin_game();

    while (app.isOpen()) {

		 clicked = false;
		 mousePos = sf::Mouse::getPosition(app);

		 // #1 Event handling
         while(app.pollEvent(event)) {
			 switch (event.type) {

			 	case sf::Event::Closed:
                    thread_kernel.detach();
			 		app.close();
                    return;

			 	case sf::Event::KeyPressed: {
			 		if(event.key.code == sf::Keyboard::Escape) {
                        thread_kernel.detach();
                        soundManager.stop_music("main_music.wav");
                        return;
                    }
			 	   	break;
			    }

		 	    case sf::Event::MouseButtonPressed : {
			 	   if (event.mouseButton.button == sf::Mouse::Left) {
						clicked = true;
                        try {
                            gameManager.handle_mouse_click(mousePos);
                        } catch (const EndOfGame& e) {
                            thread_kernel.detach();
                            soundManager.stop_music("main_music.wav");
                            return;
                        }
			 	   }
			 	   break;
                }

			 	default:
			 		break;
			 }
		 }

		 gameManager.wait_action_kernel();

		 // #2 Various objects update. Object in the foreground should be updated last
		 theBackground.update(app);

         gameManager.update(app, mousePos);

         menu.update(app, mousePos, clicked);
		 m_trophiesDrawer.update(app);

         // #3 Screen display
         app.display();
     }

     thread_kernel.join();
}
