#include "GUI.hpp"

using namespace std;

void GUI::run_game_option_loop() {

	int numOption = 1; //counter
	int yPos;
	int buttonCount;

	// _______ Background _______
	Background theBackground("menu_background.png");

	// _______ Message vector _______
	vector<Message> messageVector;

	// _______ Title _______
	messageVector.emplace_back(WINDOW_WIDTH/2, WINDOW_HEIGHT/10, TITLE_FONT_SIZE, "optionGame", "GreatVibes-Regular.ttf", WHITE);

	// _______ Button to return to main menu _______
	TextButton backButton(WINDOW_WIDTH/2, WINDOW_HEIGHT-100, MENU_BUTTON_FONT_SIZE, "back", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);

	// ________ Difficulty settings ________
	yPos = OPTION_SPACE_UNDER_TITLE+numOption*OPTION_OFFSET;
	numOption++;

	messageVector.emplace_back(GAME_OPTION_VERTICAL_LINE_POS, yPos, OPTION_FONT_SIZE, "difficultyOption", "MPlantin_0.ttf", WHITE, Align::RIGHT);

	buttonCount = 0;
	vector<TextButton> argDifficulty;
	argDifficulty.emplace_back(GAME_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*120*SCALE_RESOLUTION, yPos, OPTION_FONT_SIZE , "easy", "MPlantin_0.ttf", BLACK, GRAY_LIT, GRAY);
	argDifficulty.emplace_back(GAME_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*120*SCALE_RESOLUTION, yPos, OPTION_FONT_SIZE, "medium", "MPlantin_0.ttf", BLACK, GRAY_LIT, GRAY);
	argDifficulty.emplace_back(GAME_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*120*SCALE_RESOLUTION, yPos, OPTION_FONT_SIZE, "hard", "MPlantin_0.ttf", BLACK, GRAY_LIT, GRAY);

	LinkedButtons toggleDifficulty(argDifficulty);

	switch (optionManager()->get_difficulty()) {
		case Difficulty::EASY:
			break;
		case Difficulty::HARD:
			toggleDifficulty.change_selected(2);
			break;
		case Difficulty::MEDIUM:
		default:
			toggleDifficulty.change_selected(1);
			break;
	}

	// ________ Darken Unplayable Cards Option ________
	yPos = OPTION_SPACE_UNDER_TITLE+numOption*OPTION_OFFSET;
	numOption++;

	messageVector.emplace_back(GAME_OPTION_VERTICAL_LINE_POS, yPos, OPTION_FONT_SIZE, "darkenOption", "MPlantin_0.ttf", WHITE, Align::RIGHT);

	buttonCount = 0;
	vector<TextButton> argDarken;
	argDarken.emplace_back(GAME_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*100*SCALE_RESOLUTION, yPos, OPTION_FONT_SIZE , "on", "MPlantin_0.ttf", BLACK, GRAY_LIT, GRAY);
	argDarken.emplace_back(GAME_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*100*SCALE_RESOLUTION, yPos, OPTION_FONT_SIZE, "off", "MPlantin_0.ttf", BLACK, GRAY_LIT, GRAY);
	LinkedButtons toggleDarken(argDarken);

	if (not optionManager()->get_darken_non_playable())
		toggleDarken.change_selected(1);

    // ________ Auto play last card ________
	yPos = OPTION_SPACE_UNDER_TITLE+numOption*OPTION_OFFSET;
	numOption++;

	messageVector.emplace_back(GAME_OPTION_VERTICAL_LINE_POS, yPos, OPTION_FONT_SIZE, "autoplayOption", "MPlantin_0.ttf", WHITE, Align::RIGHT);

	buttonCount = 0;
	vector<TextButton> argAuto;
	argAuto.emplace_back(GAME_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*100*SCALE_RESOLUTION, yPos, OPTION_FONT_SIZE , "on", "MPlantin_0.ttf", BLACK, GRAY_LIT, GRAY);
	argAuto.emplace_back(GAME_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*100*SCALE_RESOLUTION, yPos, OPTION_FONT_SIZE, "off", "MPlantin_0.ttf", BLACK, GRAY_LIT, GRAY);
	LinkedButtons toggleAutoplay(argAuto);

	if (not optionManager()->get_autoplay_last())
		toggleAutoplay.change_selected(1);

    // ________ Sort by Trump Option ________
	yPos = OPTION_SPACE_UNDER_TITLE+numOption*OPTION_OFFSET;
	numOption++;

	messageVector.emplace_back(GAME_OPTION_VERTICAL_LINE_POS, yPos, OPTION_FONT_SIZE, "trumpSortOption", "MPlantin_0.ttf", WHITE, Align::RIGHT);

	buttonCount = 0;
	vector<TextButton> argTrumpSort;
	argTrumpSort.emplace_back(GAME_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*100*SCALE_RESOLUTION, yPos, OPTION_FONT_SIZE , "on", "MPlantin_0.ttf", BLACK, GRAY_LIT, GRAY);
	argTrumpSort.emplace_back(GAME_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*100*SCALE_RESOLUTION, yPos, OPTION_FONT_SIZE, "off", "MPlantin_0.ttf", BLACK, GRAY_LIT, GRAY);
	LinkedButtons toggleTrumpSort(argTrumpSort);

	if (not optionManager()->get_sorted_by_trump())
		toggleTrumpSort.change_selected(1);

	// Event handling
	sf::Event event;

	// Cursor
	sf::Vector2i mousePos;
	bool clicked = false;

	while (app.isOpen()) {

		 clicked = false;
		 mousePos = sf::Mouse::getPosition(app);

		 // #1 Event handling
		 while(app.pollEvent(event)) {

			 switch (event.type) {
				case sf::Event::Closed:
					app.close();
					return;

				case sf::Event::KeyPressed: {
					if(event.key.code == sf::Keyboard::Escape) {
						goto exportLabel;
						return;
					}
					break;
				}

			   case sf::Event::MouseButtonPressed :
				   if (event.mouseButton.button == sf::Mouse::Left) {
						clicked = true;
						if (backButton.is_lit()) {
							soundManager.play_sound("click.wav");
							exportLabel:
							switch (toggleDifficulty.who_is_selected()) {
								case 0:
									optionManager()->set_difficulty(Difficulty::EASY);
									break;
								case 2:
									optionManager()->set_difficulty(Difficulty::HARD);
									break;
								case 1:
								default:
									optionManager()->set_difficulty(Difficulty::MEDIUM);
									break;
							}
							optionManager()->set_darken_non_playable(1-toggleDarken.who_is_selected());
							optionManager()->set_autoplay_last(1 - toggleAutoplay.who_is_selected());
							optionManager()->set_sorted_by_trump(1 - toggleTrumpSort.who_is_selected());
							return; // exits the option_loop to get back to main menu
						}

				   }
				   break;

				default:
					break;
			 }
		 }

		 // #2 Various objects update
		 theBackground.update(app);
		 backButton.update(app, mousePos);
		 toggleDifficulty.update(app, mousePos, clicked);
		 toggleDarken.update(app, mousePos, clicked);
		 toggleAutoplay.update(app, mousePos, clicked);
		 toggleTrumpSort.update(app, mousePos, clicked);

		 for (auto& message : messageVector) {
			message.update(app);
		 }
		 m_trophiesDrawer.update(app);

		 // #3 Screen display
		 app.display();
	 }
}
