#include "GUI.hpp"

using namespace std;

void GUI::run_graphic_option_loop() {

	int numOption = 1; //counter
	int yPos;
	int buttonCount;

	// _______ Background _______
	Background theBackground("menu_background.png");

	// _______ Message vector _______
	vector<Message> messageVector;

	// _______ Title _______
	messageVector.emplace_back(WINDOW_WIDTH/2, WINDOW_HEIGHT/10, TITLE_FONT_SIZE, "optionGraphic", "GreatVibes-Regular.ttf", WHITE);

	// _______ Warning message _______
	messageVector.emplace_back(WINDOW_WIDTH/2,WINDOW_HEIGHT-150, OPTION_FONT_WARNING_SIZE, "optionWarning", "MPlantin_0.ttf", RED);

	// _______ Button to return to main menu _______
	TextButton backButton(WINDOW_WIDTH/2, WINDOW_HEIGHT-100, MENU_BUTTON_FONT_SIZE, "back", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);

	// ________ Fullscreen Option ________
	yPos = OPTION_SPACE_UNDER_TITLE+numOption*OPTION_OFFSET;
	numOption++;

	messageVector.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS, yPos, OPTION_FONT_SIZE, "fullscreenOption", "MPlantin_0.ttf", WHITE, Align::RIGHT);

	buttonCount = 0;
	vector<TextButton> argFullscreen;
	argFullscreen.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*100*SCALE_RESOLUTION, yPos, OPTION_FONT_SIZE , "on", "MPlantin_0.ttf", BLACK, GRAY_LIT, GRAY);
	argFullscreen.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*100*SCALE_RESOLUTION, yPos, OPTION_FONT_SIZE, "off", "MPlantin_0.ttf", BLACK, GRAY_LIT, GRAY);
	LinkedButtons toggleFullscreen(argFullscreen);

	if (not optionManager()->is_full_screen())
		toggleFullscreen.change_selected(1);


	// _____ Resolution Option ______
	yPos = OPTION_SPACE_UNDER_TITLE+numOption*OPTION_OFFSET;
	numOption++;
	messageVector.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS, yPos, OPTION_FONT_SIZE, "resolutionOption", "MPlantin_0.ttf", WHITE, Align::RIGHT);

	vector<TextButton> argResolution;
	for (int k = 0 ; k < NB_RESOLUTION ; k++)
		argResolution.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS+(150*WINDOW_WIDTH)/1920+k*(280*WINDOW_WIDTH)/1920, yPos, OPTION_FONT_SIZE , RESOLUTION[k], "MPlantin_0.ttf", BLACK, GRAY_LIT, GRAY);
	LinkedButtons chooseResolution(argResolution);

	switch (optionManager()->get_screen_width()) { // lock the correct button according to settings
		case 800:
			chooseResolution.change_selected(0);
			break;
		case 1600:
			chooseResolution.change_selected(2);
			break;
		case 1920:
			chooseResolution.change_selected(3);
			break;
		case 1080:
		default:
			chooseResolution.change_selected(1);
			break;
	}


	// _____ Language option ______
	yPos = OPTION_SPACE_UNDER_TITLE+numOption*OPTION_OFFSET;
	numOption++;

	messageVector.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS, yPos, OPTION_FONT_SIZE, "languageOption", "MPlantin_0.ttf", WHITE, Align::RIGHT);

	buttonCount = 0;
	vector<GraphicButton> argLanguage;
	argLanguage.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*(100*WINDOW_WIDTH)/1080, yPos, "flags/gb_tr.png", "flags/gb.png", "flags/gb.png", FLAG_SIZE, FLAG_SIZE);
	argLanguage.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*(100*WINDOW_WIDTH)/1080, yPos, "flags/fr_tr.png", "flags/fr.png", "flags/fr.png", FLAG_SIZE, FLAG_SIZE);
	argLanguage.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*(100*WINDOW_WIDTH)/1080, yPos, "flags/ger_tr.png", "flags/ger.png", "flags/ger.png", FLAG_SIZE, FLAG_SIZE);
	argLanguage.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*(100*WINDOW_WIDTH)/1080, yPos, "flags/esp_tr.png", "flags/esp.png", "flags/esp.png", FLAG_SIZE, FLAG_SIZE);
	argLanguage.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*(100*WINDOW_WIDTH)/1080, yPos, "flags/ita_tr.png", "flags/ita.png", "flags/ita.png", FLAG_SIZE, FLAG_SIZE);
	argLanguage.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS+100+(buttonCount++)*(100*WINDOW_WIDTH)/1080, yPos, "flags/por_tr.png", "flags/por.png", "flags/por.png", FLAG_SIZE, FLAG_SIZE);

	LinkedButtons chooseLanguage(argLanguage);
	switch (optionManager()->get_language()) { // lock the correct button according to settings
		case Language::ENGLISH:
			chooseLanguage.change_selected(0);
			break;
		case Language::GERMAN:
			chooseLanguage.change_selected(2);
			break;
		case Language::SPANISH:
			chooseLanguage.change_selected(3);
			break;
		case Language::ITALIAN:
			chooseLanguage.change_selected(4);
			break;
		case Language::PORTUGUESE:
			chooseLanguage.change_selected(5);
			break;
		case Language::FRENCH:
		default: // and pirate
			chooseLanguage.change_selected(1);
			break;
	}

	// ________ Sound Volume Option ________
	yPos = OPTION_SPACE_UNDER_TITLE+numOption*OPTION_OFFSET;
	numOption++;

	messageVector.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS, yPos, OPTION_FONT_SIZE, "soundOption", "MPlantin_0.ttf", WHITE, Align::RIGHT);

	SliderButton soundButton(GRAPHIC_OPTION_VERTICAL_LINE_POS+50, yPos+OPTION_FONT_SIZE/2, 10, 300*SCALE_RESOLUTION, sf::Color(255,255,255), "MPlantin_0.ttf", optionManager()->get_sound_volume()/100.);

	// ________ Music Volume Option ________
	yPos = OPTION_SPACE_UNDER_TITLE+numOption*OPTION_OFFSET;
	numOption++;

	messageVector.emplace_back(GRAPHIC_OPTION_VERTICAL_LINE_POS, yPos, OPTION_FONT_SIZE, "musicOption", "MPlantin_0.ttf", WHITE, Align::RIGHT);

	SliderButton musicButton(GRAPHIC_OPTION_VERTICAL_LINE_POS+50, yPos+OPTION_FONT_SIZE/2, 10, 300*SCALE_RESOLUTION, sf::Color(255,255,255), "MPlantin_0.ttf", optionManager()->get_music_volume()/100.);


	// Event handling
	sf::Event event;

	// Cursor
	sf::Vector2i mousePos;
	bool clicked = false;

	while (app.isOpen()) {

		 clicked = false;
		 mousePos = sf::Mouse::getPosition(app);

		 // #1 Event handling
		 while(app.pollEvent(event)) {

			 switch (event.type) {
				case sf::Event::Closed:
					app.close();
					return;

				case sf::Event::KeyPressed: {
					if(event.key.code == sf::Keyboard::Escape) {
						goto exportLabel;
						return;
					}
					break;
				}

			   case sf::Event::MouseButtonPressed :
				   if (event.mouseButton.button == sf::Mouse::Left) {
						clicked = true;
						if (backButton.is_lit()) {
							soundManager.play_sound("click.wav");
							exportLabel:
							// ______ Export Fullscreen mode ______
							switch (toggleFullscreen.who_is_selected()) {
								case 0: // On
									optionManager()->set_full_screen(true);
									break;
								case 1: // Off
								default:
									optionManager()->set_full_screen(false);
									break;
							}

							// ______ Export Resolution ______
							optionManager()->set_screen_width(RESOLUTION_WIDTH[chooseResolution.who_is_selected()]);
							optionManager()->set_screen_height(RESOLUTION_HEIGHT[chooseResolution.who_is_selected()]);
							optionManager()->set_sound_volume((int)100*soundButton.get_value());
							optionManager()->set_music_volume((int)100*musicButton.get_value());

							// ______ Export Language ________
							Language l;
							switch (chooseLanguage.who_is_selected()) {
								case 0:
									l = Language::ENGLISH;
									break;
								case 2:
									l = Language::GERMAN;
									break;
								case 3:
									l = Language::SPANISH;
									break;
								case 4:
									l = Language::ITALIAN;
									break;
								case 5:
									l = Language::PORTUGUESE;
									break;
								case 1:
								default:
									l = Language::FRENCH;
									break;
							}

							optionManager()->set_language(l);
							return; // exits the option_loop to get back to main menu
						}

				   }
				   break;

				default:
					break;
			 }
		 }

		 // #2 Various objects update
		 theBackground.update(app);
		 backButton.update(app, mousePos);

		 for (auto& message : messageVector) {
		 	message.update(app);
		 }

		 toggleFullscreen.update(app, mousePos, clicked);
		 chooseResolution.update(app, mousePos, clicked);
		 chooseLanguage.update(app, mousePos, clicked);
		 soundButton.update(app, sf::Mouse::isButtonPressed(sf::Mouse::Left), mousePos);
		 musicButton.update(app, sf::Mouse::isButtonPressed(sf::Mouse::Left), mousePos);
		 m_trophiesDrawer.update(app);

		 // #3 Screen display
		 app.display();
	}

}
