#include "GUI.hpp"
#include "TextEntry.hpp"
#include "Constants.hpp"
#include "Message.hpp"
#include "Button.hpp"
#include "Loading.hpp"
#include "GraphicTimer.hpp"

#include <string>

enum class LoginState : int {
    WAITING,
    TRY_LOGIN,
    LOGIN_FAIL,
    LOGIN_SUCCESS
};

void GUI::run_login(){
    sf::Time const MAX_LOGIN_RUNTIME = sf::seconds(1);

    LoginState currentState(LoginState::WAITING);

    // Background
    Background theBackground("menu_background.png");

    // Title message
    Message title(WINDOW_WIDTH/2, WINDOW_HEIGHT/10, TITLE_FONT_SIZE, "title", "GreatVibes-Regular.ttf", WHITE, Align::CENTER, sf::Text::Bold);

    // Other messages
    Message username(WINDOW_WIDTH/2, WINDOW_HEIGHT/2-100-LOGIN_FONT_SIZE, LOGIN_FONT_SIZE, "username", "MPlantin_0.ttf", WHITE, Align::CENTER);
    Message password(WINDOW_WIDTH/2, WINDOW_HEIGHT/2-LOGIN_FONT_SIZE, LOGIN_FONT_SIZE, "password", "MPlantin_0.ttf", WHITE, Align::CENTER);
    Message unableToLogin(WINDOW_WIDTH/2, WINDOW_HEIGHT/2+5*LOGIN_FONT_SIZE, LOGIN_FAIL_FONT_SIZE, "loginFail", "MPlantin_0.ttf", RED_LIT, Align::CENTER);

    // Login entries
    TextEntry userEntry(sf::Vector2f(WINDOW_WIDTH/2-9*30*0.56f,WINDOW_HEIGHT/2-100), 18, "MPlantin_0.ttf", 30, sf::Color::White); // 9*30*0.56f for centering
    TextEntry pswrdEntry(sf::Vector2f(WINDOW_WIDTH/2-9*30*0.56f,WINDOW_HEIGHT/2), 18, "MPlantin_0.ttf", 30, sf::Color::White);
    pswrdEntry.hide_text(); // Password characters replaced with *
    userEntry.set_focus(true);

    // Buttons
    TextButton offlineMode(WINDOW_WIDTH/3, WINDOW_HEIGHT-170*SCALE_RESOLUTION_HEIGHT, MENU_BUTTON_FONT_SIZE, "offlineMode", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);
    TextButton login(2*WINDOW_WIDTH/3, WINDOW_HEIGHT-170*SCALE_RESOLUTION_HEIGHT, MENU_BUTTON_FONT_SIZE, "connect", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);
    TextButton exitGame(WINDOW_WIDTH/2, WINDOW_HEIGHT-100*SCALE_RESOLUTION_HEIGHT, MENU_BUTTON_FONT_SIZE, "exit", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);

    Loading loginLoading(WINDOW_WIDTH/2-Loading::SPRITE_OFFSET-40, WINDOW_HEIGHT/2+5*LOGIN_FONT_SIZE);

    // Event handling
    sf::Event event;

    sf::Vector2i mousePos;
    sf::Clock clk;

    // utility booleans

    while (app.isOpen()) {
         mousePos = sf::Mouse::getPosition(app);

         theBackground.update(app);

         username.update(app);
         userEntry.update(app);

         password.update(app);
         pswrdEntry.update(app);

         title.update(app);

         exitGame.update(app, mousePos);
         offlineMode.update(app, mousePos);
         login.update(app, mousePos);

         m_splashScreen.update(app);

         switch (currentState) {

             case LoginState::TRY_LOGIN:
                while(app.pollEvent(event)) {
                    if (event.type == sf::Event::Closed) {
                        app.close();
                    }
                }
                loginLoading.update(app);
                if (clk.getElapsedTime() > MAX_LOGIN_RUNTIME){
                    // FOR NOW, WE LOGIN WITHOUT AUTHENTIFICATION
                    currentState = LoginState::LOGIN_SUCCESS;
                    std::string name = userEntry.get_text();
                    if (name.size() > 0) { // check for non empty names
                        optionManager()->set_player_name(userEntry.get_text());
                        optionManager()->set_online(true);
                        std::cout << "[LOGIN] Logged in with player name '" << name << "'" << std::endl;
                        clk.restart();
                    } else {
                        currentState = LoginState::LOGIN_FAIL;
                    }
                }
                break;


             case LoginState::LOGIN_SUCCESS:
                while(app.pollEvent(event)) {
                    if (event.type == sf::Event::Closed) app.close();
                }
                if (not m_splashActive){
                    m_splashScreen.fade(255,FADE_SPEED);
                    m_splashActive=true;
                }
                if (clk.getElapsedTime() > sf::seconds(SPLASH_TIME+1/FADE_SPEED)){
                    run_main_menu(); // logged in
                }
                break;

            case LoginState::LOGIN_FAIL:
               unableToLogin.update(app);

             case LoginState::WAITING:
             default: // default is WAITING
                 while(app.pollEvent(event)) {

                     userEntry.process_event(event);
                     pswrdEntry.process_event(event);

                     // check the type of the event...
                     switch (event.type) {
                        // window closed
                        case sf::Event::Closed:
                            app.close();

                        // key pressed
                        case sf::Event::KeyPressed: {
                            if(event.key.code == sf::Keyboard::Escape) {
                                app.close();
                            }
                            else if (event.key.code == sf::Keyboard::Tab) {
                                userEntry.toggle_focus();
                                pswrdEntry.toggle_focus();
                            }
                            else if (event.key.code == sf::Keyboard::Return){
                                goto try_login;
                            }
                            break;
                        }

                       case sf::Event::MouseButtonPressed:
                           if (event.mouseButton.button == sf::Mouse::Left) {
                               if (offlineMode.is_lit()) {
                                   soundManager.play_sound("click.wav");
                                   optionManager()->set_online(false);
                                   std::cout << "[LOGIN] Logged in offline mode " << std::endl;
                                   currentState = LoginState::LOGIN_SUCCESS;
                                   clk.restart();
                               } else if (login.is_lit()) {
                                   soundManager.play_sound("click.wav");
                                   try_login:
                                   currentState=LoginState::TRY_LOGIN;
                                   clk.restart();
                               } else if (exitGame.is_lit()) {
                                   soundManager.play_sound("click.wav");
                                   app.close();
                               }
                           }
                           break;

                        // we don't process other types of events
                        default:
                            break;
                     }
                 }

         } // end SWITCH

    app.display();

    } // end WHILE
}
