#include "GUI.hpp"
#include "TextManager.hpp"

using namespace std;

void GUI::run_main_menu() {

    if (optionManager()->is_online()){
        statManager.load_profile(optionManager()->get_player_name());
    } else {
        statManager.load_profile("_offline");
    }

	// Background
	Background theBackground("menu_background.png");

    // Title message
	Message title(WINDOW_WIDTH/2, WINDOW_HEIGHT/10, TITLE_FONT_SIZE, "title", "GreatVibes-Regular.ttf", WHITE, Align::CENTER, sf::Text::Bold);

    // Buttons of main menu
	TextButton startGame(WINDOW_WIDTH/2, MENU_SPACE_UNDER_TITLE+MENU_OFFSET, MENU_BUTTON_FONT_SIZE, "play", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);
	TextButton tuto(WINDOW_WIDTH/2, MENU_SPACE_UNDER_TITLE+2*MENU_OFFSET, MENU_BUTTON_FONT_SIZE, "tuto", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);
	TextButton options(WINDOW_WIDTH/2, MENU_SPACE_UNDER_TITLE+3*MENU_OFFSET, MENU_BUTTON_FONT_SIZE, "option", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);
	TextButton stats(WINDOW_WIDTH/2, MENU_SPACE_UNDER_TITLE+4*MENU_OFFSET, MENU_BUTTON_FONT_SIZE, L"Statistiques", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);
	TextButton credits(WINDOW_WIDTH/2, MENU_SPACE_UNDER_TITLE+5*MENU_OFFSET, MENU_BUTTON_FONT_SIZE, "credit", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);
	TextButton exitGame(WINDOW_WIDTH/2, MENU_SPACE_UNDER_TITLE+6*MENU_OFFSET, MENU_BUTTON_FONT_SIZE, "exit", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);

	//load text
	textManager.load("trophies/"+ optionManager()->get_language_string()+".txt");

	// Event handling
	sf::Event event;

	// Cursor
	sf::Vector2i mousePos;

	bool ready = false; // ready = not m_splashActive and not splash_screen.is_fading()
	sf::Clock clk;

    while (app.isOpen()) {

		 mousePos = sf::Mouse::getPosition(app);

		 // #1 Event handling
         while(app.pollEvent(event)) {
			 // check the type of the event...
			 switch (event.type) {
			 	// window closed
			 	case sf::Event::Closed:
			 		app.close();

			 	// key pressed
			 	case sf::Event::KeyPressed: {
			 		if(event.key.code == sf::Keyboard::Escape)
						app.close();
			 	   	break;
			    }

		 	   case sf::Event::MouseButtonPressed :
			 	   if (event.mouseButton.button == sf::Mouse::Left) {
			 		   	if (startGame.is_lit() and ready) {
                            soundManager.play_sound("click.wav");
                        	run_mode_selection_loop();
					   	}

						else if (tuto.is_lit() and ready) {
                            soundManager.play_sound("click.wav");
							run_tutorial_menu();
						}

						else if (options.is_lit() and ready) {
                            soundManager.play_sound("click.wav");
							run_options_loop();
						}

						else if (stats.is_lit() and ready) {
                            soundManager.play_sound("click.wav");
							run_stat_loop();
							break;
						}

						else if (exitGame.is_lit() and ready) {
                            soundManager.play_sound("click.wav");
							app.close();
					 		break;
						}

						else if (credits.is_lit() and ready) {
                            soundManager.play_sound("click.wav");
							run_credits_loop();
						}

			 	   }
			 	   break;

			 	// we don't process other types of events
			 	default:
			 		break;
			 }
		 }

		 if (m_splashActive){
			 m_splashScreen.fade(0,FADE_SPEED);
			 m_splashActive = false;
		 }
		 if (clk.getElapsedTime() > sf::seconds(1/FADE_SPEED)) {
			 ready = true;
		 }

		 // #2 Various objects update
		 theBackground.update(app);

		 title.update(app);

		 startGame.update(app, mousePos);
		 tuto.update(app, mousePos);
		 options.update(app, mousePos);
		 credits.update(app, mousePos);
		 stats.update(app, mousePos);
		 exitGame.update(app, mousePos);
		 m_trophiesDrawer.update(app);

		 m_splashScreen.update(app);

         // #3 Screen display
         app.display();
     }
}
