#include <thread>

#include "GUI.hpp"
#include "IA.hpp"
#include "Core.hpp"

#include "BetArray.hpp"
#include "DropDownMenu.hpp"
#include "GraphicGame.hpp"

#include <iostream>
using namespace std;

void GUI::run_multiplayer_game_loop(ConnectionType const conType, short const serverID) {
    try {
        soundManager.play_music("main_music.wav");
        Core::ParallelPlayer * gui_player = new Core::ParallelPlayer();
        shared_ptr<Network::RemoteKernel> kernel;
        m_serverLink.init();
        switch (conType) {
            case ConnectionType::CREATE: {
                kernel.reset(m_serverLink.create_game(gui_player));
                break;
            }
            case ConnectionType::JOIN: {
                kernel.reset(m_serverLink.join_game(gui_player, serverID));
                break;
            }
            case ConnectionType::MATCHMAKING: {
                kernel.reset(m_serverLink.join_matchmaking(gui_player));
                break;
            }
            default:
                throw std::string("[multiplayerGameLoop.cpp] ERROR : ConnectionType not recognized");
                break;
        }

    	// Temporary. TODO : add a button to start the game ?
    	m_serverLink.start_game();

        thread thread_kernel(&Network::RemoteKernel::run, kernel);

        GraphicGame gameManager(&kernel->get_game_state(), gui_player);

    	Background theBackground("tapis.png");

    	DropDownMenu menu(&kernel->get_game_state());

    	// Event handling
    	sf::Event event;

    	// Cursor
    	sf::Vector2i mousePos;
    	bool clicked  = false;

        cout << "[MultiplayerGameLoop] init done" << endl;
        while (app.isOpen()) {

    		 clicked = false;
    		 mousePos = sf::Mouse::getPosition(app);

    		 // #1 Event handling
             while(app.pollEvent(event)) {
    			 switch (event.type) {

    			 	case sf::Event::Closed:
    			 		app.close();
                        return;

    			 	case sf::Event::KeyPressed: {
    			 		if(event.key.code == sf::Keyboard::Escape) {
                            soundManager.stop_music("main_music.wav");
                            kernel->stop();
                            thread_kernel.detach();
                            return;
                        }
    			 	   	break;
    			    }

    		 	    case sf::Event::MouseButtonPressed : {
    			 	   if (event.mouseButton.button == sf::Mouse::Left) {
    						clicked = true;
    						gameManager.handle_mouse_click(mousePos);
    			 	   }
    			 	   break;
                    }

    			 	default:
    			 		break;
    			 }
    		 }

    		 gameManager.wait_action_kernel();

    		 // #2 Various objects update. Object in the foreground should be updated last
    		 theBackground.update(app);
    		 gameManager.update(app, mousePos);
    		 menu.update(app, mousePos, clicked);
    		 m_trophiesDrawer.update(app);

             // #3 Screen display
             app.display();
         }

         thread_kernel.join();
    } catch (std::runtime_error& e) {
        std::cerr << "[multiplayerGameLoop] received runtime error" << std::endl;
    }

}
