#include "GUI.hpp"
#include "TextEntry.hpp"
#include "Loading.hpp"
#include "Message.hpp"
#include "Button.hpp"

#include <string>

using namespace Network;

enum class LobbyState : int {
    WAITING,
    LOGIN,
    LOGIN_FAIL,
};

void GUI::run_multiplayer_lobby(){
    LobbyState currentState(LobbyState::WAITING);

    // Background
    Background theBackground("menu_background.png");

    // Other messages
    Message serverID(WINDOW_WIDTH/2, WINDOW_HEIGHT/2-50-LOGIN_FONT_SIZE, LOGIN_FONT_SIZE, L"Identifiant serveur", "MPlantin_0.ttf", WHITE, Align::CENTER);
    Message error(WINDOW_WIDTH/2, WINDOW_HEIGHT/2+5*LOGIN_FONT_SIZE, LOGIN_FAIL_FONT_SIZE, L"erreur", "MPlantin_0.ttf", RED_LIT, Align::CENTER);

    // Entry
    TextEntry serverEntry(sf::Vector2f(WINDOW_WIDTH/2-9*30*0.56f,WINDOW_HEIGHT/2-50), 18, "MPlantin_0.ttf", 30, sf::Color::White); // 9*30*0.56f for centering
    serverEntry.set_focus(true);

    // Buttons
    TextButton login(WINDOW_WIDTH/2, WINDOW_HEIGHT/2 + 50*SCALE_RESOLUTION_HEIGHT, MENU_BUTTON_FONT_SIZE, "connect", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);
    TextButton back(WINDOW_WIDTH/2, WINDOW_HEIGHT - 150*SCALE_RESOLUTION_HEIGHT, MENU_BUTTON_FONT_SIZE, "back", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);

    Loading loginLoading(WINDOW_WIDTH/2-Loading::SPRITE_OFFSET-40, WINDOW_HEIGHT/2+5*LOGIN_FONT_SIZE);

    // Event handling
    sf::Event event;

    sf::Vector2i mousePos;
    sf::Clock clk;

    // utility booleans
    bool runLoop = true;

    while (runLoop and app.isOpen()) {
         mousePos = sf::Mouse::getPosition(app);

         theBackground.update(app);

         serverID.update(app);
         serverEntry.update(app);

         back.update(app, mousePos);
         login.update(app, mousePos);

         switch (currentState) {

             case LobbyState::LOGIN: {
                while(app.pollEvent(event)) {
                    if (event.type == sf::Event::Closed) app.close();
                }
                short serverID = (short)stoi(std::string(serverEntry.get_text()));
                try {
                    run_multiplayer_game_loop(ConnectionType::JOIN, serverID);
                    runLoop=false; // At the end of multiplayer loop, go back to multiplayer selection menu
                } catch (Status stat) {
                    currentState = LobbyState::LOGIN_FAIL;
                    switch (stat) {
                    	case Status::Full:
                            error.set_text(L"This server is full");
                            break;
                        case Status::NotFound:
                            error.set_text(L"Error : This server does not seem to exist");
                            break;
                        case Status::Error:
                            error.set_text(L"Connexion error");
                            break;
                        case Status::Done: // nothing particular
                        default:
                            break;
                    }
                }
                break;
            }

             case LobbyState::LOGIN_FAIL:
                error.update(app);

             case LobbyState::WAITING:
             default: // default is WAITING
                 while(app.pollEvent(event)) {

                     serverEntry.process_event(event);

                     // check the type of the event...
                     switch (event.type) {
                        // window closed
                        case sf::Event::Closed:
                            app.close();

                        // key pressed
                        case sf::Event::KeyPressed: {
                            if(event.key.code == sf::Keyboard::Escape) {
                                app.close();
                            }
                            else if (event.key.code == sf::Keyboard::Return){
                                goto try_login;
                            }
                            break;
                        }

                       case sf::Event::MouseButtonPressed:
                           if (event.mouseButton.button == sf::Mouse::Left) {
                               if (login.is_lit()) {
                                   soundManager.play_sound("click.wav");
                                   try_login:
                                   currentState=LobbyState::LOGIN;
                               } else if (back.is_lit()) {
                                   return;
                               }
                           }
                           break;

                        // we don't process other types of events
                        default:
                            break;
                     }
                 }

         } // end SWITCH

    m_trophiesDrawer.update(app);

    // Screen display
    app.display();

    } // end WHILE
}
