#include "GUI.hpp"
#include "Constants.hpp"

#include <iostream>

void GUI::choose_functionnality(Function const f) {
    switch (f) {
        case Function::NONE:
            break;
        case Function::OPTIONS_LOOP:
            run_options_loop();
            break;
        case Function::GRAPHIC_OPTION_LOOP:
            run_graphic_option_loop();
            break;
        case Function::GAME_OPTION_LOOP:
            run_game_option_loop();
            break;
        case Function::CREDITS:
            run_credits_loop();
            break;
        case Function::MODE_SELECTION:
            run_mode_selection_loop();
            break;
        case Function::MULTIPLAYER_SELECTION:
            run_multiplayer_selection();
            break;
        case Function::TUTORIAL_SELECTION:
            run_tutorial_menu();
            break;
        case Function::GAME_LOOP:
            run_game_loop();
            break;
        case Function::TUTORIAL_CARDS_LOOP:
            run_tutorial_card_loop();
            break;
        case Function::TUTORIAL_TRICKS_LOOP:
            run_tutorial_loop("tuto_trick");
            break;
        case Function::TUTORIAL_BET_LOOP:
            run_tutorial_loop("tuto_bet");
            break;
        case Function::TUTORIAL_FLOW_LOOP:
            run_tutorial_loop("tuto_flow");
            break;
        case Function::MULTI_CREATE_GAME:
            run_multiplayer_game_loop(ConnectionType::CREATE);
            break;
        case Function::MULTI_JOIN_GAME:
            run_multiplayer_lobby();
            break;
        case Function::MULTI_FIND_GAME:
            run_multiplayer_game_loop(ConnectionType::MATCHMAKING);
            break;
        default:
            break;
    }
}

void GUI::run_loop(std::vector<Message>  &messages,
              std::vector<TextButton> &text_buttons,
              std::vector<GraphicButton> &graphic_buttons,
              std::vector<Function> &functionnalities) {

    // Background
    Background theBackground("menu_background.png");

    // Event handling
    sf::Event event;

    // Cursor
    sf::Vector2i mousePos;

    while (app.isOpen()) {
        mousePos = sf::Mouse::getPosition(app);

        // #1 Event handling
        while(app.pollEvent(event)) {
            // check the type of the event...
            switch (event.type) {
            // window closed
                case sf::Event::Closed:
                    app.close();

                // key pressed
                case sf::Event::KeyPressed: {
                    if(event.key.code == sf::Keyboard::Escape)
                        return;
                    break;
                }

                case sf::Event::MouseButtonPressed :
                    if (event.mouseButton.button == sf::Mouse::Left) {
                        for (unsigned int i = 0 ; i < text_buttons.size() ; i++) {
                            if (text_buttons[i].is_lit()) {
                                soundManager.play_sound("click.wav");
                                if (functionnalities[i]==Function::QUIT)
                                    return;
                                choose_functionnality(functionnalities[i]);
                            }
                        for (unsigned int i = 0 ; i < graphic_buttons.size() ; i++) {
                            if (graphic_buttons[i].is_lit()) {
                                soundManager.play_sound("click.wav");
                                if (functionnalities[i]==Function::QUIT)
                                    return;
                                choose_functionnality(functionnalities[i]);
                            }
                        }
                    }
                    break;

                // we don't process other types of events
                default:
                    break;
                }
            }
        }

        // #2 Various objects update
        theBackground.update(app);
        m_trophiesDrawer.update(app);

        for (auto &m : messages)
            m.update(app);

        for (auto &b : text_buttons)
            b.update(app, mousePos);

        for (auto &b : graphic_buttons)
            b.update(app, mousePos);

        // #3 Screen display
        app.display();
    }
}
