#include "GUI.hpp"
#include "TextManager.hpp"

using namespace std;

void GUI::run_stat_loop() {

	// _______ Background _______
	Background theBackground("menu_background.png");

	// _______ Message vector _______
	vector<Message> messageVector;

	// _______ Title _______
	messageVector.emplace_back(WINDOW_WIDTH/2, WINDOW_HEIGHT/10, TITLE_FONT_SIZE, "statTrophy", "GreatVibes-Regular.ttf", WHITE);

	// _______ Button to return to main menu _______
	TextButton backButton(WINDOW_WIDTH/2, WINDOW_HEIGHT-100, MENU_BUTTON_FONT_SIZE, "back", "MPlantin_0.ttf", GRAY, GRAY_LIT, BLACK);

	textManager.load("stats/"+ optionManager()->get_language_string()+".txt");
	auto &stat_vector = statManager.get_stats();

	for(unsigned int i=0; i<NB_STATS; i++){
	    messageVector.emplace_back(WINDOW_WIDTH/3+20*SCALE_RESOLUTION, WINDOW_HEIGHT/8 + TITLE_FONT_SIZE + i*(STAT_FONT_SIZE+3), STAT_FONT_SIZE, "stat"+to_string(i), "MPlantin_0.ttf", WHITE, Align::LEFT);
	    messageVector.emplace_back(WINDOW_WIDTH/3, WINDOW_HEIGHT/8 + TITLE_FONT_SIZE +  i*(STAT_FONT_SIZE+3) , STAT_FONT_SIZE, to_wstring(stat_vector[i]), "MPlantin_0.ttf", WHITE, Align::RIGHT);
	}

	// Event handling
	sf::Event event;

	// Cursor
	sf::Vector2i mousePos;

	while (app.isOpen()) {

		 mousePos = sf::Mouse::getPosition(app);

		 // #1 Event handling
		 while(app.pollEvent(event)) {

			 switch (event.type) {
				case sf::Event::Closed:
					app.close();
					return;

				case sf::Event::KeyPressed: {
					if(event.key.code == sf::Keyboard::Escape) {
						return;
					}
					break;
				}

			   case sf::Event::MouseButtonPressed :
				   if (event.mouseButton.button == sf::Mouse::Left) {
                        if (backButton.is_lit()) {
							soundManager.play_sound("click.wav");
							return;
						}

				   }
				   break;

				// we don't process other types of events
				default:
					break;
			 }
		 }

		 // #2 Various objects update
		 theBackground.update(app);
		 backButton.update(app, mousePos);

		 for (auto& message : messageVector) {
		 	message.update(app);
		 }
		 m_trophiesDrawer.update(app);

		 // #3 Screen display
		 app.display();
	 }

}
