#include "GUI.hpp"
#include "TutoKernel.hpp"
#include "TutoBubble.hpp"

void unlock_mutex(){
    std::unique_lock<std::mutex> lock(TutoKernel::okMutex);
    TutoKernel::okReady = true;
	TutoKernel::okVariable.notify_one();
}

void GUI::run_tutorial_card_loop(){
    bool label2_end = false;

    Core::GameState gamestate(0);
    Core::ParallelPlayer player;

    TutoBubble tutoMessage(sf::Vector2f(WINDOW_WIDTH/2,WINDOW_HEIGHT/2),L"",PointPosition::TOP_RIGHT);
    auto tutoKernel = std::make_shared<TutoKernel>("tuto_cards", nullptr, nullptr, nullptr);
    std::thread thread_tuto(&run_tuto,tutoKernel);

    std::array<Hand,4> hand = { Hand(WINDOW_WIDTH/2,WINDOW_HEIGHT/2-2*LENGTH_CARD*SCALE_RESOLUTION_HEIGHT,0,1.2),
                                Hand(WINDOW_WIDTH/2,WINDOW_HEIGHT/2-LENGTH_CARD*SCALE_RESOLUTION_HEIGHT,0,1.2),
                                Hand(WINDOW_WIDTH/2,WINDOW_HEIGHT/2,0,1.2),
                                Hand(WINDOW_WIDTH/2,WINDOW_HEIGHT/2+LENGTH_CARD*SCALE_RESOLUTION_HEIGHT,0,1.2)};
    for(int value=0; value<9; value++){
        for(int suit=0; suit<4; suit++){
            hand[suit].give_card(Core::Card(static_cast<Core::Suit> (suit+1), static_cast<Core::Value> (value)));
        }
    }

    for(int suit=0; suit<4; suit ++)
        hand[suit].replace_card(30);

    // Background
	Background theBackground("tapis.png");

    // Event handling
	sf::Event event;
    sf::Vector2i mousePos;

    while (app.isOpen()) {

        mousePos = sf::Mouse::getPosition(app);

		 // #1 Event handling
         while(app.pollEvent(event)) {
			 // check the type of the event...
			 switch (event.type) {
			 	// window closed
			 	case sf::Event::Closed:
			 	    tutoKernel->close();
			 	    thread_tuto.detach();
			 		app.close();
                    return;

                // key pressed
                case sf::Event::KeyPressed: {
                    if(event.key.code == sf::Keyboard::Escape){
                        tutoKernel->close();
                        thread_tuto.detach();
                        return;
                    } else if(event.key.code == sf::Keyboard::Space){
                        // same action as clicking tutoMessage button
                        goto next_event_tutoCardLoop;
                        return;
                    }
                    break;
                }

			 	case sf::Event::MouseButtonPressed :
                    if (event.mouseButton.button == sf::Mouse::Left) {
				 	    if(tutoMessage.is_ok_active() && tutoMessage.button_lit()){
                            soundManager.play_sound("click.wav");
                            next_event_tutoCardLoop:
				 	        unlock_mutex();
				 	    }
                    }
				 	break;

			 	// we don't process other types of events
			 	default:
			 		break;
			 }
		 }

		bool pause;
        std::string messageKey;
        bool newMessage;
        sf::Vector2f posMessage;
        tutoKernel->getMessage(messageKey,newMessage,pause, posMessage);
		 if(newMessage){
		    std::cout << messageKey << "\n";
		    if(messageKey=="label2"){
		        label2_end = true;
		        unlock_mutex();

		    } else if(messageKey=="label1"){
		        hand[0].set_playable_cards(std::vector<Core::Card> (1,Core::Card(Core::Suit::Diamond, Core::Value::Cat)));
		        hand[2].set_playable_cards(std::vector<Core::Card> (1,Core::Card(Core::Suit::Diamond, Core::Value::Cat)));
		        hand[3].set_playable_cards(std::vector<Core::Card> (1,Core::Card(Core::Suit::Diamond, Core::Value::Cat)));
		        unlock_mutex();

		    } else if(messageKey=="label3"){
		        Core::Hand blinkCard;
		        blinkCard.emplace_back(Core::Suit::Diamond, Core::Value::King);
		        blinkCard.emplace_back(Core::Suit::Diamond, Core::Value::Queen);
		        blinkCard.emplace_back(Core::Suit::Diamond, Core::Value::Knight);
		        blinkCard.emplace_back(Core::Suit::Diamond, Core::Value::Knave);
		        hand[1].set_playable_cards(blinkCard);
		        unlock_mutex();

		    } else if(messageKey=="label4"){
		        Core::Hand blinkCard;
		        blinkCard.emplace_back(Core::Suit::Diamond, Core::Value::Entertainer);
		        blinkCard.emplace_back(Core::Suit::Diamond, Core::Value::Musician);
		        blinkCard.emplace_back(Core::Suit::Diamond, Core::Value::Juggler);
		        hand[1].set_playable_cards(blinkCard);
		        unlock_mutex();

		    } else if(messageKey=="label5"){
		        Core::Hand blinkCard;
		        blinkCard.emplace_back(Core::Suit::Diamond, Core::Value::Dog);
		        blinkCard.emplace_back(Core::Suit::Diamond, Core::Value::Cat);
		        hand[1].set_playable_cards(blinkCard);
		        unlock_mutex();

		    } else if(messageKey=="label6"){
		        hand[1].sort_hand(Core::Trump::Diamond);
		        hand[1].set_playable_cards(Core::Hand());
		        unlock_mutex();
		    } else if(messageKey=="label7"){


            } else {
		        tutoMessage.set_text(messageKey);
		        tutoMessage.set_ok_active(pause);
		        tutoMessage.set_position(posMessage);
		    }
        }

		 if(tutoKernel->tuto_ending()){
		    thread_tuto.join();
		    return;
		 }

		 // #2 Various objects update. Object in the foreground should be updated last
		 theBackground.update(app);

		 for(int suit=0; suit<4; suit ++)
		    if(!label2_end || suit==1)
                hand[suit].draw(app);
         tutoMessage.update(app, mousePos);
         m_trophiesDrawer.update(app);
         // #3 Screen display
         app.display();
    }

}
