#include "GUI.hpp"
#include "TutoKernel.hpp"
#include "TutoBubble.hpp"

void GUI::run_tutorial_loop(std::string name){

    Core::GameState gamestate(0);
    Core::ParallelPlayer player;

    TutoBubble tutoMessage(sf::Vector2f(WINDOW_WIDTH/2,WINDOW_HEIGHT/2),L"",PointPosition::TOP_RIGHT);

    GraphicGame graphicGame(&gamestate, &player);
    auto tutoKernel = std::make_shared<TutoKernel>(name, &gamestate, &player, &graphicGame);
    std::thread thread_tuto(&run_tuto,tutoKernel);

    // Background
	Background theBackground("tapis.png");

    // Event handling
	sf::Event event;
    sf::Vector2i mousePos;

    while (app.isOpen()) {

        mousePos = sf::Mouse::getPosition(app);

		 // #1 Event handling
         while(app.pollEvent(event)) {
			 // check the type of the event...
			 switch (event.type) {
			 	// window closed
			 	case sf::Event::Closed:
			 	    tutoKernel->close();
			 	    thread_tuto.detach();
			 		app.close();
                    return;

                // key pressed
                case sf::Event::KeyPressed: {
                    if(event.key.code == sf::Keyboard::Escape){
                        tutoKernel->close();
                        thread_tuto.detach();
                        return;
                    } else if(event.key.code == sf::Keyboard::Space){
                        // same action as clicking tutoMessage button
                        goto next_event_tutoLoop;
                        return;
                    }
                    break;
                }

			 	case sf::Event::MouseButtonPressed :
				    if (event.mouseButton.button == sf::Mouse::Left) {
                        graphicGame.handle_mouse_click(mousePos);
				 	}
				 	if(tutoMessage.is_ok_active() && tutoMessage.button_lit()){
                        soundManager.play_sound("click.wav");
                        next_event_tutoLoop:
				 	    std::unique_lock<std::mutex> lock(TutoKernel::okMutex);
				 	    TutoKernel::okReady = true;
				 	    TutoKernel::okVariable.notify_one();
				 	}
				 	break;

			 	// we don't process other types of events
			 	default:
			 		break;
			 }
		 }

        bool pause;
        std::string messageKey;
        bool newMessage;
        sf::Vector2f posMessage;
        tutoKernel->getMessage(messageKey,newMessage,pause,posMessage);
		 if(newMessage){
		    tutoMessage.set_text(messageKey);
		    tutoMessage.set_ok_active(pause);
		    tutoMessage.set_position(posMessage);
        }

		 if(!graphicGame.is_animated() && tutoKernel->tuto_ending()){
		    thread_tuto.join();
		    statManager.win_trophy(TROPHY::END_TUTO);
		    return;
		 }

		 graphicGame.wait_action_kernel();
		 // #2 Various objects update. Object in the foreground should be updated last
		 theBackground.update(app);
         graphicGame.update(app, mousePos);
         tutoMessage.update(app, mousePos);
         m_trophiesDrawer.update(app);
         // #3 Screen display
         app.display();
    }

}
