
#include "AnimatedTransformable.hpp"

using namespace sf;

//________________________Constructor____________________________

AnimatedTransformable::AnimatedTransformable() {
	m_destination = getPosition();
	m_angleDestination = 0;
	m_scaleDestination = 1;
	m_speed = 1;
	m_speedScale = 0.01;
	m_speedAngle = 1;
}

//________________________Animation_Function_____________________

void AnimatedTransformable::continue_move(){
	double delta_x = m_destination.x - getPosition().x;
	double delta_y = m_destination.y - getPosition().y;
	double distance = abs(delta_x) + abs(delta_y);

	double delta_angle = m_angleDestination - getRotation();
	if(delta_angle > 0){
	    delta_angle -=360;
	}
	if(delta_angle <= -180){
	    delta_angle += 360; // on tourne dans l'autre sens
	}

	if(distance >=1){
		if(distance >= m_speed){
		    double facteur = m_speed / distance;
		    delta_x *= facteur;
		    delta_y *= facteur;
		}
		move(delta_x, delta_y);
    }

	if(delta_angle > m_speedAngle){
		delta_angle = m_speedAngle;
	}
	else if(delta_angle < -m_speedAngle){
		delta_angle = -m_speedAngle;
	}
	rotate(delta_angle);

	double delta_scale = m_scaleDestination - getScale().y;
	//std::cout << delta_scale << " " << getScale().y << std::endl;
	if(delta_scale >= 0.01 || delta_scale <= -0.01){
		if(delta_scale > 0){
			delta_scale = std::min(delta_scale,  m_speedScale);
		} else {
			delta_scale = std::max(delta_scale, -m_speedScale);
		}
		setScale(getScale().x + delta_scale,getScale().y + delta_scale);
	}
}

//_________________________Move_Function___________________________

void AnimatedTransformable::move_sprite(int x, int y, int angle, double speed){
	move_sprite(Vector2f(x,y), angle, speed, speed);
}

void AnimatedTransformable::move_sprite(sf::Vector2f position, int angle, double speed, double speedAngle){
	m_destination = position;
	m_speed = speed;
	if(m_speed < 1){
	    m_speed = 1;
	}
	m_speedAngle = speedAngle;

	if(angle < 0){
	    angle += 360;
	}
	m_angleDestination = angle;
}

void AnimatedTransformable::move_sprite(sf::Vector2i position, int angle, double speed){
	move_sprite(Vector2f(position), angle, speed, speed);
}

//_________Move_Relative__________

void AnimatedTransformable::move_sprite_relative(int x, int y, int angle, int speed){
    move_sprite(x+getPosition().x,y+getPosition().y,angle+getRotation(),speed);
}

void AnimatedTransformable::move_sprite_relative(sf::Vector2f position, int angle, int speed){
    move_sprite(position+getPosition(), angle+getRotation(), speed, speed);
}

void AnimatedTransformable::move_sprite_relative(sf::Vector2i position, int angle, int speed){
	move_sprite(sf::Vector2f(position)+getPosition(), angle+getRotation(), speed, speed);
}

//________Move_in_time_________

void AnimatedTransformable::move_sprite_in_time(int x, int y, int angle, int time){
    move_sprite_in_time(sf::Vector2f(x,y),angle, time);
}

void AnimatedTransformable::move_sprite_in_time(sf::Vector2f position, int angle, int time){
    int delta_x = position.x - getPosition().x;
	int delta_y = position.y - getPosition().y;
	int distance = abs(delta_x) + abs(delta_y);

	if(angle < 0){
	    angle += 360;
	}
	int angleDist = getRotation() - angle;
	if(angleDist < 0){
	    angleDist += 360;
	}

    move_sprite(position,angle,double(distance)/double(time),double(angleDist)/double(time));
}

void AnimatedTransformable::move_sprite_in_time(sf::Vector2i position, int angle, int time){
	move_sprite_in_time(sf::Vector2f(position), angle, time);
}

//_________Other_move_________

void AnimatedTransformable::set_position(int x, int y, int angle){
    set_position(sf::Vector2f(x,y),angle);
}
void AnimatedTransformable::set_position(sf::Vector2f position, int angle){
    if(angle < 0){
	    angle += 360;
	}
    setPosition(position);
    setRotation(angle);
    m_destination = position;
    m_angleDestination = angle;
}
void AnimatedTransformable::set_position(sf::Vector2i position, int angle){
    set_position(sf::Vector2f(position),angle);
}

void AnimatedTransformable::zoom_sprite(double scale, double time){
	m_scaleDestination = scale;
	if(scale - getScale().y > 0){
	    m_speedScale = ( scale - getScale().y )/time;
	}
	else{
	    m_speedScale = ( getScale().y - scale )/time;
	}
}

void AnimatedTransformable::set_scale(double scale){
	m_scaleDestination = scale;
	setScale(scale,scale);
}

void AnimatedTransformable::set_angle(int angle){
    m_angleDestination = angle;
    setRotation(angle);
}


void AnimatedTransformable::setSpeed(int speed){
    m_speed = speed;
    if(m_speed < 1){
        m_speed = 1;
    }
    m_speedAngle = speed;
}

//_________________________Event_Function___________________________

bool AnimatedTransformable::is_animated(){
    int delta_x = m_destination.x - getPosition().x;
	int delta_y = m_destination.y - getPosition().y;
	int distance = abs(delta_x) + abs(delta_y);

	double delta_angle = m_angleDestination - getRotation();
	if(delta_angle == 360){
	    delta_angle = 0;
	}
	double delta_scale = m_scaleDestination - getScale().x;

	if(distance >=1 || abs(delta_angle) >= 1 || delta_scale > 0.01 || delta_scale < -0.01){ //if the sprite moves, don't touch
	    return true;
	}
	return false;
}

//_________________________Debug_Function____________________________

void AnimatedTransformable::debug(){
    std::cout << m_destination.x << " "<< m_destination.y << " " <<
	m_angleDestination << " " <<
	m_scaleDestination << " " <<
	m_speed << " " <<
	m_speedScale << " " <<
	m_speedAngle << " " << std::endl;
	std::cout << getPosition().x << " "<< getPosition().y << " " <<
	getRotation() << " " <<
	getScale().x << " " <<
	getScale().y <<  std::endl;
}
