#include "OptionManager.hpp"
#include "SoundManager.hpp"

using namespace std;

// Global option manager
OptionManager* optionManager() {
	static unique_ptr<OptionManager> x{new OptionManager()};
	return x.get();
}

// _____ Constructor and destructor ______
OptionManager::OptionManager() :
 	m_online(false)
{
	ifstream optionRead;
	optionRead.open("options.txt");
	cout << "[OptionManager] Options at start:\n";
	string output;

	// __________ GRAPHICAL OPTIONS __________
	getline(optionRead,output);
	m_isFullScreen = stoi(output);
	m_fullScreenVideoMode = sf::VideoMode::getFullscreenModes()[0]; // mode 0 gives the highest resolution

	getline(optionRead, output);
	int screenWidth = stoi(output);

	getline(optionRead, output);
	int screenHeight = stoi(output);

	sf::VideoMode desktop = sf::VideoMode::getDesktopMode();
	m_normalVideoMode = sf::VideoMode(screenWidth, screenHeight, desktop.bitsPerPixel);

	getline(optionRead,output);
	m_gameLanguage = string_to_language(output); // default is FRENCH

	getline(optionRead,output);
	m_soundVolume = stoi(output);

	getline(optionRead,output);
	m_musicVolume = stoi(output);

	getline(optionRead,output);
	m_defaultIP = sf::IpAddress(output);

	getline(optionRead, output);
	m_defaultPort = stoi(output);

	if (m_isFullScreen)
		cout << "  Fullscreen " << m_fullScreenVideoMode.width << " " << m_fullScreenVideoMode.height << " " << m_fullScreenVideoMode.bitsPerPixel <<"bpi\n ";
	cout << "  Resolution : " << screenWidth << " " << screenHeight << "\n";
	cout << "  Language : " << language_to_string(m_gameLanguage) << "\n";
	cout << "  Sound Volume : " << m_soundVolume << "/100\n";
	cout << "  Music Volume : " << m_musicVolume << "/100\n";
	cout << "  Default server IP : " << m_defaultIP << endl;
	cout << "  Default server port : " << m_defaultPort << endl;

	// ___________ GAME OPTIONS ____________

	getline(optionRead,output);
	m_difficulty = string_to_difficulty(output);
	cout << "  Difficulty mode : " << output << "\n";

	getline(optionRead,output);
	m_darkenNonPlayable = (bool)stoi(output);
	cout << "  Darken non playable cards : "<< m_darkenNonPlayable <<"\n";

	getline(optionRead,output);
	m_autoPlayLast = (bool)stoi(output);
	cout << "  Auto play last card : "<< m_autoPlayLast << "\n";

	getline(optionRead,output);
	m_trumpSort = (bool)stoi(output);
	cout << "  Sort hand according to trump : " << m_trumpSort << "\n";

	cout << endl;
	optionRead.close();
}

OptionManager::~OptionManager() {
    ofstream optionWrite;
    optionWrite.open("options.txt");
	cout << endl;
	cout << "[OptionManager] Options at the end of the execution :\n";

	optionWrite << m_isFullScreen <<"\n";
	cout << (m_isFullScreen ? "  Fullscreen " : "");

    optionWrite << m_normalVideoMode.width <<"\n";
	cout << "  Resolution : " << m_normalVideoMode.width <<" ";

	optionWrite << m_normalVideoMode.height <<"\n";
	cout << m_normalVideoMode.height <<"\n";

	string strLanguage = language_to_string(m_gameLanguage);
	optionWrite << strLanguage+"\n";
	cout << "  Language : " << strLanguage << endl;

	optionWrite << m_soundVolume << "\n";
	cout << "  Sound Volume : " << m_soundVolume << "/100\n";

	optionWrite << m_musicVolume << "\n";
	cout << "  Music Volume : " << m_musicVolume << "/100\n";

	optionWrite << m_defaultIP.toString() << "\n";
	cout << "  Default server IP : " << m_defaultIP.toString() << endl;

	optionWrite << m_defaultPort << endl;
	cout << "  Default server port : " << m_defaultPort << endl;

	string strDifficulty = difficulty_to_string(m_difficulty);
	optionWrite << strDifficulty +"\n";
	cout << "  Difficulty mode : "<< strDifficulty << endl;

	optionWrite << m_darkenNonPlayable << "\n";
	cout << "  Darken non playable cards : "<< m_darkenNonPlayable <<"\n";

	optionWrite << m_autoPlayLast << "\n";
	cout << "  Auto play last card : "<< m_autoPlayLast << "\n";

	optionWrite << m_trumpSort << "\n";
	cout << "  Sort hand according to trump : " << m_trumpSort<< "\n";

    optionWrite.close();
}

// _____ Getters ______
int OptionManager::get_screen_width() const {
	if (m_isFullScreen)
		return m_fullScreenVideoMode.width;
	return m_normalVideoMode.width;
}

int OptionManager::get_screen_height() const {
	if (m_isFullScreen)
		return m_fullScreenVideoMode.height;
	return m_normalVideoMode.height;
}

bool OptionManager::is_full_screen() const {
	return m_isFullScreen;
}

Language OptionManager::get_language() const {
    return m_gameLanguage;
}

bool OptionManager::is_pirate() const {
	return m_gameLanguage==Language::PIRATE;
}

std::string OptionManager::get_language_string() const {
    return language_to_string(m_gameLanguage);
}

sf::VideoMode OptionManager::get_window_settings() const {
	if (m_isFullScreen)
		return m_fullScreenVideoMode;
	return m_normalVideoMode;
}

int OptionManager::get_sound_volume() const {
	return m_soundVolume;
}

int OptionManager::get_music_volume() const {
	return m_musicVolume;
}

sf::IpAddress OptionManager::get_default_server_IP(){
	return m_defaultIP;
}

int OptionManager::get_default_server_port() {
	return m_defaultPort;
}

Difficulty OptionManager::get_difficulty() const {
	return m_difficulty;
}

bool OptionManager::get_darken_non_playable() const {
	return m_darkenNonPlayable;
}

bool OptionManager::get_autoplay_last() const {
	return m_autoPlayLast;
}

bool OptionManager::get_sorted_by_trump() const {
	return m_trumpSort;
}

bool OptionManager::is_online() const{
	return m_online;
}

sf::String OptionManager::get_player_name() const {
	return m_playerName;
}

// _____ Setters ______
void OptionManager::set_screen_width(int width) {
    m_normalVideoMode.width = width;
}

void OptionManager::set_screen_height(int height) {
    m_normalVideoMode.height = height;
}

void OptionManager::set_full_screen(bool x) {
	m_isFullScreen = x;
}

void OptionManager::set_language(Language lang) {
    m_gameLanguage = lang;
}

void OptionManager::set_sound_volume(int const volume) {
	m_soundVolume = volume;
	soundManager.set_sound_volume(volume);
}

void OptionManager::set_music_volume(int const volume) {
	m_musicVolume = volume;
	soundManager.set_music_volume(volume);
}

void OptionManager::set_server_IP(sf::IpAddress const ip) {
	m_defaultIP = ip;
}

void OptionManager::set_server_port(int const port){
	m_defaultPort=port;
}

void OptionManager::set_difficulty(Difficulty d) {
	m_difficulty=d;
}

void OptionManager::set_darken_non_playable(bool dark) {
	m_darkenNonPlayable = dark;
}

void OptionManager::set_autoplay_last(bool a) {
	m_autoPlayLast = a;
}

void OptionManager::set_sorted_by_trump(bool sort) {
	m_trumpSort = sort;
}

void OptionManager::set_online(bool online){
	m_online = online;
}

void OptionManager::set_player_name(std::string const& s){
	m_playerName = s;
}
