#include "StatManager.hpp"
#include "OptionManager.hpp"
#include "SoundManager.hpp"

using namespace std;

StatManager statManager; // Global StatManager

//________________________________________
StatManager::StatManager(): m_hasLoaded(false) {}

void StatManager::load_profile(std::string const name){
    ifstream file("res/profile/"+name+".profile");
    m_localStats = vector<int>(NB_STATS,0);
    m_trophies = vector<int>(static_cast<int> (TROPHY::NB_TROPHIES),0);
    if(file.is_open()){
        std::cout << "[StatManager] Loading profile '"<< name << "'\n";
        for(unsigned int i=0; i<m_localStats.size(); i++) {
            file >> m_localStats[i];
        }
        for(unsigned int i=0; i<m_trophies.size(); i++) {
            file >> m_trophies[i];
        }
        m_profileName = name;
        m_hasLoaded = true;
    } else {
        std::cout << "[StatManager] Unable to find profile '" << name << "'. Statistics won't be saved" << std::endl;
    }
    if(optionManager()->get_language() == Language::PIRATE) {
        win_trophy(TROPHY::DISCOVER_PIRATE_LANGUAGE);
    }
}

void StatManager::save_profile(std::string const name){
    if (m_hasLoaded) {
        ofstream file("res/profile/"+name+".profile");
        if(file.is_open()){
            std::cout << "[StatManager] Saving profile '"<< name << "'\n";
            for(unsigned int i=0; i<m_localStats.size(); i++) {
                file << m_localStats[i] << "\n";
            }
            for(unsigned int i=0; i<m_trophies.size(); i++) {
                file << m_trophies[i] << "\n";
            }
        } else {
            throw runtime_error("[StatManager] Unable to open res/profile/"+name+".profile");
        }
    } else {
        std::cout << "[StatManager] No profile file loaded. Terminating." << std::endl;
    }

}

StatManager::~StatManager() {
    save_profile(m_profileName);
}

//_________________________________________

void StatManager::begin_game(){
    auto &s = m_localStats;
    s[NB_GAME]++;
    s[GAME_LEFT]++;
}

void StatManager::begin_round(const Core::GameState &gs){
    auto &s = m_localStats;

    s[NB_ROUNDS]++;
    if(gs.is_choune()){
        s[NB_MECHOUNE_ROUND]++;
        s[NB_CHOUNE_ROUND]++;
        m_bufMechoune = gs.id_mechouner();
        m_bufChoune = gs.id_chouner();
    } else if(gs.is_mechoune()) {
        s[NB_MECHOUNE_ROUND]++;
        m_bufMechoune = gs.id_mechouner();
        m_bufChoune = -1;
    } else {
        m_bufMechoune = -1;
        m_bufChoune = -1;
    }

    m_bufScores = gs.get_scores();
}

void StatManager::end_game(const Core::GameState &gs){
    auto &s = m_localStats;

    auto scores = gs.get_scores();
    int rank=4;
    auto last = max_element(scores.begin(), scores.end());
    while(last != scores.begin()+gs.my_id()){
        rank--;
        *last = -1;
        last = max_element(scores.begin(), scores.end());
    }

    if(s[BEST_RANK]==0 || rank < s[BEST_RANK])
        s[BEST_RANK] = rank;
    if(s[BEST_SCORE]==0 || scores[gs.my_id()] < s[BEST_SCORE])
        s[BEST_SCORE] = scores[gs.my_id()];
    s[GAME_LEFT]--;
    s[GAME_FIRST + rank -1]++;

    win_trophy(TROPHY::END_A_GAME);
    if(rank==4)
        win_trophy(TROPHY::LOSE_A_GAME);
    else if(rank==1){
        win_trophy(TROPHY::WIN_A_GAME);
        if(s[NB_GAME]>=10)
            win_trophy(TROPHY::WIN_TEN_GAME);
        if(s[NB_GAME]>=100)
            win_trophy(TROPHY::WIN_HUNDRED_GAME);
    }
    if(scores[gs.my_id()]==0)
        win_trophy(TROPHY::ZERO_POINTS_GAME);
}

void StatManager::end_round(const Core::GameState &gs){
    auto &s = m_localStats;
    auto scores = gs.get_scores();
    for(int i=0; i<4; i++)
        scores[i] -= m_bufScores[i];

    s[TOTAL_POINTS_RECEIVED] += scores[gs.my_id()];
    s[AVERAGE_ROUND_POINTS_RECEIVED] = s[TOTAL_POINTS_RECEIVED] / s[NB_ROUNDS];
    s[AVERAGE_ROUND_TRICK] = s[NB_TRICKS] / s[NB_ROUNDS];

    if(scores[gs.my_id()] <= 5)
        win_trophy(static_cast<TROPHY> ( static_cast<int> (TROPHY::ZERO_POINTS_ROUND) + scores[gs.my_id()] / (m_bufMechoune!=-1 ? (m_bufChoune!=-1 ? 4 : 2) : 1)));

    if(m_bufMechoune==gs.my_id() && scores[gs.my_id()]){
        win_trophy(TROPHY::FALL_WHEN_YOUR_MECHOUNE);

    }
    if(m_bufChoune==gs.my_id() && scores[gs.my_id()])
        win_trophy(TROPHY::FALL_WHEN_YOUR_CHOUNE);

    if(scores[0] && scores[1] && scores[2] && scores[3])
        win_trophy(TROPHY::ALL_PLAYERS_FALL);
}

void StatManager::end_trick(const Core::GameState &gs){
    auto &s = m_localStats;
    auto &stack = gs.last_card_stack();
    if(stack.id_winner() == gs.my_id()){
        s[NB_TRICKS]++;
        if(stack.strongest_card().value() == Core::Value::Cat){
            win_trophy(TROPHY::TAKE_TRICK_WITH_CAT);
            if(stack.strongest_card().suit() == Core::Suit::Club)
                s[NB_TRICK_CAT_CLUB]++;
        }
        if(stack.strongest_card().value() == Core::Value::Dog){
            for(auto &card : stack.stack()){
                if(card.value() == Core::Value::Cat && card.suit() == stack.strongest_card().suit())
                    win_trophy(TROPHY::DOG_ON_CAT);
            }
        }
    } else {
        if(stack.get_card(gs.my_id()).value() == Core::Value::King)
            win_trophy(TROPHY::LOSE_TRICK_WITH_KING);
    }
}

//_____________________________________________

bool StatManager::new_trophy(){
    return m_bufTrophies.size()!=0;
}

void StatManager::win_trophy(TROPHY trophy){
    if(m_trophies[ static_cast<int> (trophy)])
        return;
    m_trophies[static_cast<int> (trophy)] = 1;
    m_bufTrophies.push(trophy);
    if (optionManager()->is_pirate()){
        soundManager.play_sound("42.ogg");
    } else {
        soundManager.play_sound("tada.ogg");
    }
}

TROPHY StatManager::get_next_trophy(){
    auto temp = m_bufTrophies.front();
    m_bufTrophies.pop();
    return temp;
}

//_________________________________________

std::vector<int>& StatManager::get_stats(){
    return m_localStats;
}

std::vector<int>& StatManager::get_trophies(){
    return m_trophies;
}
