#include "TextManager.hpp"
#include <fstream>
#include <locale>
#include <iostream>

using namespace std;

TextManager textManager; // Global TextManager

TextManager::TextManager() {
    load(optionManager()->get_language_string() + ".txt");
}

void TextManager::load(std::string const &fileName){
    
#if __APPLE__
    #define LOCALE_FR "fr_FR.UTF-8"
    #define LOCALE_US "en_US.UTF-8"
#else
    #define LOCALE_FR "fr_FR.utf8"
    #define LOCALE_US "en_US.utf8"
#endif

    try {
        std::locale::global(std::locale(LOCALE_FR));
    } catch (const std::exception& e) {
        cout << "[TextManager] Warning : locale " << LOCALE_FR << " not found" << endl;
        std::locale::global(std::locale(LOCALE_US));
    }

    wcout.imbue(locale());

    cout << "[TextManager] Loading strings from file " << fileName << endl;
    wifstream textfile;
    textfile.open("res/text/"+fileName);

    wstring key;
    wstring value;
    while (textfile >> key) {
        if (!getline(textfile,value)){
            cerr << "[TextManager] Error while loading file '" << fileName << "'\n";
        }
        m_dataContainer[sf::String(key).toAnsiString()]=value;
    }
    textfile.close();
    std::locale::global(std::locale("C"));
}

wstring TextManager::get_entry(string const &key) {
    if (m_dataContainer.find(key) == m_dataContainer.end() ) {
        cerr << "[TextManager] " << key << " : Invalid key for TextManager.\n";
        return L"STRING NOT FOUND";
    }
    return m_dataContainer[key];
}
