#include "CryptoBox.hpp"

namespace Network{
    CryptoBox::CryptoBox(bool is_client){
        m_isClient = is_client;
        cerr << "[CryptoBox] Initialisation of the crypto protocols" << endl;

        if(sodium_init() < 0){
            cerr << "[CryptoBox] The sodium library could not be initialized properly. Aborting mission." << endl;
			throw(std::runtime_error("Sodium lib could not initialize"));
        }

        /* Generate the client's key pair */
        crypto_kx_keypair(m_public_key, m_private_key);

		m_nonce = 0;
    }

    void CryptoBox::init(unsigned char* distant_public_key){
        //cerr << "Initialisation of the crypto"
		/* Check the exchanged keys and build the crypto keys*/
		int status;
		if(m_isClient){
    		cerr << "[CryptoBox] Initialization of client keys" << endl;
    		status = crypto_kx_client_session_keys(m_receive_key, m_send_key,
            	m_public_key, m_private_key, distant_public_key);
		} else {
    		cerr << "[CryptoBox] Initialization of server keys" << endl;
    		status = crypto_kx_server_session_keys(m_receive_key, m_send_key,
            	m_public_key, m_private_key, distant_public_key);
		}
		m_nonce = 0;

		if(status != 0)
        {
            /* Suspicious server public key, bail out */
            cerr << "[CryptoBox] The keys do not match, there is something strange here... Be careful" << endl;
            throw(std::runtime_error("Bad server key format"));
        }
    }

	void CryptoBox::cypher_message(unsigned char* cyphered, unsigned char* source, size_t size){
    	unsigned char nonce[crypto_secretbox_NONCEBYTES];
    	for(unsigned int i = 0; i < crypto_secretbox_NONCEBYTES; i++){
        	nonce[i] = 0;
    	}
    	memcpy(nonce, &m_nonce, sizeof(uint64_t));

        crypto_secretbox_easy(cyphered, source, size, nonce, m_send_key);
        m_nonce++;
	}

	void CryptoBox::decypher_message(unsigned char* decyphered, unsigned char* source, size_t size){
    	unsigned char nonce[crypto_secretbox_NONCEBYTES];
    	for(unsigned int i = 0; i < crypto_secretbox_NONCEBYTES; i++){
        	nonce[i] = 0;
    	}
    	memcpy(nonce, &m_nonce, sizeof(uint64_t));

        if (crypto_secretbox_open_easy(decyphered, source, size, nonce, m_receive_key) != 0){
            throw(std::runtime_error("Could not decypher. Message may be forged"));
        }

        m_nonce++;
	}

	unsigned char * CryptoBox::public_key(){
    	return m_public_key;
	}
}
