#include <SFML/Network.hpp>
#include <memory>
#include <iostream> //for debug
#include "RemoteGameState.hpp"
#include "GameMessage.hpp"

using namespace Core;

namespace Network {

    RemoteGameState::RemoteGameState(int myId, std::shared_ptr<sf::TcpSocket> s, CryptoBox * cb):
        GameState(myId),
        m_socket(s),
        m_crypto(cb)
    {
    }

    void RemoteGameState::notify_bet(const Bet& bet) {
        GameState::notify_bet(bet);
        GameMessage msg;
        msg.fill_with_notifybet(bet);
        Network::check_status(msg.send(m_socket, m_crypto));
    }

    void RemoteGameState::notify_play(const Card& card) {
        GameState::notify_play(card);
        GameMessage msg;
        msg.fill_with_notifyplay(card);
        Network::check_status(msg.send(m_socket, m_crypto));
    }

    void RemoteGameState::notify_cards(const Hand& hand) {
        GameState::notify_cards(hand);
        GameMessage msg;
        msg.fill_with_notifycards(hand);
        Network::check_status(msg.send(m_socket, m_crypto));
    }

    void RemoteGameState::notify_mechoune(int player) {
        GameState::notify_mechoune(player);
        GameMessage msg;
        msg.fill_with_notifymechoune(player);
        Network::check_status(msg.send(m_socket, m_crypto));
    }

    void RemoteGameState::notify_choune() {
        GameState::notify_choune();
        GameMessage msg;
        msg.fill_with_notifychoune();
        Network::check_status(msg.send(m_socket, m_crypto));
    }

    void RemoteGameState::notify_next_player(int player) {
        GameState::notify_next_player(player);
        GameMessage msg;
        msg.fill_with_notifynextplayer(player);
        Network::check_status(msg.send(m_socket, m_crypto));
    }

    void RemoteGameState::notify_trick_win(int player) {
        GameState::notify_trick_win(player);
        GameMessage msg;
        msg.fill_with_notifytrickwin(player);
        Network::check_status(msg.send(m_socket, m_crypto));
    }

    void RemoteGameState::notify_end_bet_state() {
        GameState::notify_end_bet_state();
        GameMessage msg;
        msg.fill_with_notifyendbetstate();
        Network::check_status(msg.send(m_socket, m_crypto));
    }

    void RemoteGameState::notify_end_card_state() {
        GameState::notify_end_card_state();
        GameMessage msg;
        msg.fill_with_notifyendcardstate();
        Network::check_status(msg.send(m_socket, m_crypto));
    }

    void RemoteGameState::notify_game_win(int player) {
        GameState::notify_game_win(player);
        GameMessage msg;
        msg.fill_with_notifygamewin(player);
        Network::check_status(msg.send(m_socket, m_crypto));
    }

    void RemoteGameState::notify_ready_player(int player) {
        GameState::notify_ready_player(player);
        GameMessage msg;
        msg.fill_with_notifyreadyplayer(player);
        Network::check_status(msg.send(m_socket, m_crypto));
    }

    void RemoteGameState::notify_begin_bet_state() {
        GameState::notify_begin_bet_state();
        GameMessage msg;
        msg.fill_with_notifybeginbetstate();
        Network::check_status(msg.send(m_socket, m_crypto));
    }
}
