#include <stdexcept>
#include <SFML/Network.hpp>

#include "GameMessage.hpp"
#include "RemotePlayer.hpp"
#include "RemoteGameState.hpp"

using namespace Core;

namespace Network {

    RemotePlayer::RemotePlayer(std::shared_ptr<sf::TcpSocket> s, CryptoBox * cb):
        m_socket(s), m_crypto(cb)
    {}


    Bet RemotePlayer::bet() {
        GameMessage msg;
        msg.fill_with_askbet();
        Network::check_status(msg.send(m_socket, m_crypto));
        wait_for_receive(m_socket);
        Network::check_status(msg.receive(m_socket, m_crypto));
        if(msg.header()==MessageHeader::Bet)
            return msg.read_bet();
        else
            throw(std::range_error("wrong message header"));
    }

    bool RemotePlayer::mechoune() {
        GameMessage msg;
        msg.fill_with_askmechoune();
        Network::check_status(msg.send(m_socket, m_crypto));
        wait_for_receive(m_socket);
        Network::check_status(msg.receive(m_socket, m_crypto));
        if(msg.header()==MessageHeader::Mechoune)
            return msg.read_bool();
        else
            throw(std::range_error("wrong message header"));
    }

    bool RemotePlayer::choune() {
        GameMessage msg;
        msg.fill_with_askchoune();
        Network::check_status(msg.send(m_socket, m_crypto));
        wait_for_receive(m_socket);
        Network::check_status(msg.receive(m_socket, m_crypto));
        if(msg.header()==MessageHeader::Choune)
            return msg.read_bool();
        else
            throw(std::range_error("wrong message header"));
    }

    Card RemotePlayer::play() {
        GameMessage msg;
        msg.fill_with_askplay();
        Network::check_status(msg.send(m_socket, m_crypto));
        wait_for_receive(m_socket);
        Network::check_status(msg.receive(m_socket, m_crypto));
        if(msg.header()==MessageHeader::Play)
            return msg.read_card();
        else
            throw(std::range_error("wrong message header"));
    }

    void RemotePlayer::set_game_state(const GameState& gameState) {
        GameMessage msg;
        m_gameState = &gameState;
        msg.fill_with_initgamestate(gameState.my_id());
        Network::check_status(msg.send(m_socket, m_crypto));
    }

    unique_ptr<GameState> RemotePlayer::create_game_state(int id) {
        return std::unique_ptr<GameState>(new RemoteGameState(id, m_socket, m_crypto));
    }
}
