#include "ServerCommunication.hpp"

using namespace std;
using namespace Core;

namespace Network{

    ServerCommunication::ServerCommunication(sf::IpAddress addr, unsigned short port):
    	m_addr(addr), m_port(port), m_timeout(timeout)
    	{
			m_socket = make_shared<sf::TcpSocket>();
			m_crypto = new CryptoBox(true);
        }

	sf::Socket::Status ServerCommunication::init(){
    	m_socket->setBlocking(true);
    	cerr << "[client] Connecting to remote server" << endl;
    	sf::Socket::Status st = m_socket->connect(m_addr, m_port, m_timeout);
		cerr << "[client] Connected to server (" << m_socket->getRemoteAddress()
			 << " : " << m_socket->getRemotePort() << ")"<< endl;

		cerr << "[client] Starting the key exchange protocol" << endl;
		unsigned char server_public_key[crypto_kx_PUBLICKEYBYTES];
		size_t received;
		// Getting the server public key
		if (m_socket->receive(server_public_key, crypto_kx_PUBLICKEYBYTES, received) != sf::Socket::Status::Done){
    		throw(std::runtime_error("Key exchange error"));
		}

		// Compute the keys from it
		m_crypto->init(server_public_key);

		// Send my public key to the server
		m_socket->send(m_crypto->public_key(), crypto_kx_PUBLICKEYBYTES);

		cerr << "[client] Crypto initialization done!" << endl;
		return st;
    }

	RemoteKernel * ServerCommunication::create_game(Core::Player * player){
		ServerMessage message;
		message.write_header(ServerHeader::CreateGame);

		// Send game creation
		message.send(m_socket, m_crypto);
		// receive answer
		message.receive(m_socket, m_crypto);

		if(message.read_status() != Status::Done)
    		throw(std::runtime_error("An error occured during game creation"));

		RemoteKernel * kernel = new RemoteKernel(player, m_socket, m_crypto);

		cerr << "[client] Kernel built!" << endl;
		return kernel;
	}

	RemoteKernel * ServerCommunication::join_game(Core::Player * player, char game_id){
		ServerMessage message;
		message.write_header(ServerHeader::JoinGame);
		message.write_lobby(game_id);

		// Send game join
		message.send(m_socket, m_crypto);
		// receive answer
		message.receive(m_socket, m_crypto);

		if(message.read_status() != Status::Done)
    		throw(std::runtime_error("An error occured during game creation"));

		RemoteKernel * kernel = new RemoteKernel(player, m_socket, m_crypto);

		cerr << "[client] Kernel built!" << endl;
		return kernel;
	}

	RemoteKernel * ServerCommunication::join_matchmaking(Core::Player * player){
		ServerMessage message;
		message.write_header(ServerHeader::FillGame);

		// Send game fill message
		message.send(m_socket, m_crypto);
		// receive answer
		message.receive(m_socket, m_crypto);

		if(message.read_status() != Status::Done)
    		throw(std::runtime_error("An error occured during game creation"));

		RemoteKernel * kernel = new RemoteKernel(player, m_socket, m_crypto);

		cerr << "[client] Kernel built!" << endl;
		return kernel;
	}

	void ServerCommunication::start_game(){
		ServerMessage message;
		message.write_header(ServerHeader::StartGame);

		message.send(m_socket, m_crypto);
    }
}
