#include "ServerMessage.hpp"

using namespace std;

namespace Network{
    ServerMessage::ServerMessage() {
        for(unsigned int i=0; i<dataSize; i++)
            m_data[i]=-1;
    }

    ServerHeader ServerMessage::header() const {
        return static_cast<ServerHeader>(m_data[0]);
    }

    unsigned char * ServerMessage::data(){
        return m_data;
    }

    void ServerMessage::receive(shared_ptr<sf::TcpSocket> socket, CryptoBox * crypto){
        unsigned char cyphered[cypheredSize];
    	if(socket->receive(cyphered, cypheredSize, m_received) != sf::Socket::Status::Done){
        	throw(std::runtime_error("An error occured during the receiving of a message"));
    	}

        crypto->decypher_message(m_data, cyphered, cypheredSize);
    }

    void ServerMessage::send(shared_ptr<sf::TcpSocket> socket, CryptoBox * crypto){
    	unsigned char cyphered[cypheredSize];
    	crypto->cypher_message(cyphered, m_data, dataSize);

		if(socket->send(cyphered, cypheredSize, m_received) != sf::Socket::Status::Done){
    		throw(std::runtime_error("Failed to send the message"));
		}
	}

    void ServerMessage::write_header(ServerHeader hdr){
        m_data[0] = static_cast<char>(hdr);
    }

	void ServerMessage::write_lobby(char id){
    	m_data[1] = id;
	}

	void ServerMessage::write_status(Status status){
    	m_data[1] = static_cast<char>(status);
	}

	char ServerMessage::read_lobby() const{
    	return m_data[1];
	}

	Status ServerMessage::read_status() const{
    	return static_cast<Status>(m_data[1]);
	}

	Status translate_status(sf::Socket::Status st){
    	switch(st){
        	case sf::Socket::Status::Done:
            	return Status::Done;
        	case sf::Socket::Status::NotReady:
        	case sf::Socket::Status::Partial:
           	case sf::Socket::Status::Disconnected:
           	default:
            	return Status::Error;
    	}
	}


}
